/****************************************************************************
   Filename : sbc-test.c
	Version	: 2.00
Description	: Hardware and software test routines for 3000 and 5000 products
		 By	: M. Johnson
	   Date : 20011130
****************************************************************************/
/*
20011130 mtj	2.00	original release of new sbc-test
20020212 mtj	2.01	added code for TS-3300, minor bug fixes
20020312 mtj	2.02	debug TS-3100 code, add code to test COM3 & 4
20020401 mtj	2.03	added debug on cmd line, added JP6 for 3300
20020403 mtj	2.03	added 3400, battery sram, atod tests
20020930 lay	2.04	split com3/4 test, changed ethernet display code
						Added code to test_DIO2_3200 to account for 
						Reset Pin of 3400
20021009 lay	2.04    new interface for setting, detecting, Options
						added test code for TS9400(bus, UserChipSelect)
						added option to enter Deubg mode from main menu
20021022 mtj	2.05	bug fixes
20021022 mtj	2.05	made RS485 test COM port selectable
20021023 mtj	2.06	added support for TS-5400 and TS-5500
20021111 mtj	2.06	added support for 4150, minor bug fixes
20021111 mtj	2.06	detects industrial temp bit and display option
20021111 mtj	2.06	support COM5 thru COM8
20030217 mtj	2.06	support for 128K SRAM on TS-3x000
20030303 mtj	2.06	support for RS-485 on TS-SER4 tests
20030516 mtj	2.07	TS-SER4 address changed
20030516 mtj	2.07	fixed set MAC for longs, then removed from sbc-test
20030804 mtj	2.08	Added support for TS-9700 and go/no go for all analog
20030812 mtj	2.08	Added support for TS-5600
20040113 mtj	2.09	Cleaning up 5600 tests, add support for TS-5700
20040114 mtj	2.09	Removed GetPswStatus func, cleaned and commented code
20040114 mtj	2.09	Added code to save hardware options in the eeprom

/*

SBC-test is used at Technologic Systems to perform hardware
and software tests prior to shipment. The product under test 
is automatically detected where possible and only the 
applicable tests are available. Many of the tests require
external loopback plugs to pass. Some tests require 
specialized external test equipment to pass.

Use the '+' key to turn on debug messages
or
Use 'D' on the command line to enable debug messages

Use 'K' on the command line to enable matrix keypad tests

This code is always under construction as we add new products.

Mike Johnson

info@embeddedx86.com

set TAB spaces to 4


	// put this anywhere in code to drop into the bios debugger
	
	asm { int 3; }

	// or type "int3" at the DOS prompt when booted to General Software DOS-ROM

*/

/****************************************************************************
	INCLUDE FILES
****************************************************************************/

#include <dos.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/****************************************************************************
	TYPEDEFS AND DEFINES
****************************************************************************/

#define FALSE 0
#define TRUE !FALSE

#define ON	1
#define OFF	0

#define  NORMAL_SRAM 0
#define BATTERY_SRAM 1

enum ErrorCodes {
	NO_ERRORS,
	NON_TS_BIOS
};

#define TS2100 21
#define TS2200 22
#define TS2350 23
#define TS2800 28
#define TS3100 31
#define TS3200 32
#define TS3300 33
#define TS3400 34
#define TS4150 41
#define TS5300 53
#define TS5400 54
#define TS5500 55
#define TS5600 56
#define TS5700 57


#define P1CFG 0xF820
#define P2CFG 0xF822
#define P3CFG 0xF824

#define P1DIR 0xF864
#define P2DIR 0xF86C
#define P3DIR 0xF874

#define P1LTC 0xF862
#define P2LTC 0xF86A
#define P3LTC 0xF872

#define P1PIN 0xF860
#define P2PIN 0xF868
#define P3PIN 0xF870

#define TS9700BASE		0x160

#define TS9400BASE		0x189

#define COM1BASE        0x3F8   /*  Base port address for COM1  */
#define COM2BASE        0x2F8   /*  Base port address for COM2  */
#define COM3BASE        0x3E8   /*  Base port address for COM3  */
#define COM4BASE        0x2E8   /*  Base port address for COM4  */
#define COM5BASE        0x3A8   /*  Base port address for COM5  */
#define COM6BASE        0x2A8   /*  Base port address for COM6  */
#define COM7BASE        0x3A0   /*  Base port address for COM7  */
#define COM8BASE        0x2A0   /*  Base port address for COM8  */

#define TSSER4BASE		0x234	/* Base port address for TS-SER4 */

#define TXR             0       /*  Transmit register (WRITE) */
#define RXR             0       /*  Receive register  (READ)  */
#define LCR             3       /*  Line control              */
#define MCR             4       /*  Modem control             */
#define LSR             5       /*  Line Status               */
#define MSR             6       /*  Modem Status              */
#define SCRATCH         7       /*  Scratch Pad Reg.          */
#define DLL             0       /*  Divisor Latch Low         */
#define DLH             1       /*  Divisor latch High        */





/*-------------------------------------------------------------------*
  Bit values held in the Line Status Register (LSR).
	 bit		meaning
	 ---		-------
	 0		Data ready.
	 1		Overrun error - Data register overwritten.
	 2		Parity error - bad transmission.
	 3		Framing error - No stop bit was found.
	 4		Break detect - End to transmission requested.
	 5		Transmitter holding register is empty.
	 6		Transmitter shift register is empty.
	 7     Time out - off line.
 *-------------------------------------------------------------------*/

#define RCVRDY          0x01
#define OVRERR          0x02
#define PRTYERR         0x04
#define FRMERR          0x08
#define BRKERR          0x10
#define XMTRDY          0x20
#define XMTRSR          0x40
#define TIMEOUT			0x80

#define DTR             0x01
#define RTS             0x02

#define CTS             0x10
#define DSR             0x20
#define RI				0x40
#define DCD				0x80

#define RTC_ADDR		0x70	// Real-Time clock address register
#define RTC_DATA		0x71	// Real-Time clock data register

#define LCD_CTRL		0x72
#define LCD_DATA		0x73

#define PLD_CLOCK_ADDR	0x70	// PLD clock selection register (R)
#define PLD_PROD_ADDR	0x74	// PLD product code register (R)
#define PLD_OPT1_ADDR	0x75	// PLD option 1 register (R)
#define PLD_OPT2_ADDR	0x76	// PLD option 2 register (R)
#define PLD_STATUS_LTC	0x77	// PLD status register for JP3,4
#define PLD_OPT3_ADDR	0x78	// PLD option 3 register (R)
#define PLD_DIO1_CTRL	0x7A	// DIO1 control register
#define PLD_DIO1_LOW	0x7B	// DIO1 Latch register, bits 0-7
#define PLD_DIO1_HIGH	0x7C	// DIO1 Latch register, bits 8-13
#define PLD_DIO2_CTRL	0x7D	// DIO2 & LCD control register
#define PLD_DIO2_LOW	0x7E	// DIO2 Latch register, bits 0-7
#define PLD_DIO2_HIGH	0x7F	// DIO2 Latch register, bits 8-13

#define PLD_SRAM_SET	0x01	// bit 0 of Option 1 for SRAM enable
#define PLD_RS485_SET	0x02	// bit 1 of Option 1 for RS-485 enable
#define PLD_ATOD_SET	0x04	// bit 2 of Option 1 for AtoD enable

#define PLD_EXTRES_SET	0x01	// bit 0 of Option 2 for external reset
#define PLD_IND_SET		0x02	// bit 1 of Option 2 for industrial temp

#define PLD_128K_SET	0x01	// bit 0 of Option 3 for 128 KB SRAM

#define PLD_MASK		0x07

#define MAC_LENGTH		6

#define PIO_0_15_PFS       0xC20
#define PIO_0_15_DIR       0xC2A
#define PIO_0_15_DATA      0xC30
#define PIO_0_15_CLEAR     0xC38
#define PIO_0_15_SET       0xC34
#define PIO_16_31_PFS      0xC22
#define PIO_16_31_DIR      0xC2C
#define PIO_16_31_DATA     0xC32
#define PIO_16_31_CLEAR    0xC3A
#define PIO_16_31_SET      0xC36

#define SPI_CTRL_REG       0xCD0
#define SPI_TX_DATA        0xCD1
#define SPI_CMD_REG        0xCD2
#define SPI_STATUS_REG     0xCD3
#define SPI_RX_DATA        0xCD4

#define CTRL_O			0x0F
#define ESC				0x1B
#define CR				0x0D

enum SPI_functions {
	EEPROM_CHIP_WE,
	EEPROM_WRITE_STATUS_REG,
	EEPROM_WRITE_OPTIONS_BYTE,
	EEPROM_READ_OPTIONS_BYTE,
	EEPROM_DUMP_DATA,
	EEPROM_SETUP_MMCR
};

// option bit defines for 5600/5700

#define OP_SRAM		0x01
#define OP_ADC		0x02
#define OP_DAC		0x04
#define OP_EXT		0x08





/***************************************************************************
	GLOBAL VARIABLES
****************************************************************************/

const char TS_STR[]         = "Technologic Systems, Inc";
const char VERSION_STR[]    = "EPC-test Ver: 2.09";

const char *ErrorMesg[] =
{
	"No Errors.",
	"Technologic Systems BIOS not detected."
};

union REGS regs;	// allows register access, used to call BIOS interrupts

unsigned char ucError = FALSE;	// error flag

unsigned char ucTemp1;
unsigned char ucTemp2;
unsigned char ucTemp3;
unsigned char ucTemp4;

unsigned long   SocketAddress = 0xD000;

struct  time t;
struct  date d;

int redirect = 0;
int iModel = 0;
int i;

char sModelNumber[8] = {"Unknown"};
unsigned char ucBiosMajor   =0;
unsigned char ucBiosMinor   =0;
unsigned int uiJumpers      =0;
unsigned int uiLowRamSize   =0;
unsigned long ulExtRamSize   =0;

struct struct_Options {
	unsigned RS485 : 1;
	unsigned ATOD : 1;
	unsigned SRAM : 1;
	unsigned ETHERNET : 1;
	unsigned COM3 : 1;
	unsigned COM4 : 1;
	unsigned COM5 : 1;
	unsigned COM6 : 1;
	unsigned COM7 : 1;
	unsigned COM8 : 1;
	unsigned EXTRESET : 1;
	unsigned Bus16bit: 1;
	unsigned TS9400 : 1;
	unsigned Keypad : 1;
	unsigned Industrial : 1;
	unsigned TSSER1 : 1;
	unsigned TSSER4 : 1;
	unsigned TSSER4_RS485 : 1;
	unsigned SRAM_128K : 1;
	unsigned TS9700 : 1;
	unsigned DAC_9700 : 1;
	unsigned DAC : 1;
	unsigned EXT : 1;
} Options ;



unsigned char ucKeyPress;

char sResult_Ethernet[13]   = {"Not Tested"};
char sResult_RTC[13]        = {"Not Tested"};
char sResult_COM1[13]       = {"Not Tested"};
char sResult_COM2[13]       = {"Not Tested"};
char sResult_COM3[13]       = {"Not Tested"};
char sResult_COM4[13]       = {"Not Tested"};
char sResult_COM5[13]       = {"Not Tested"};
char sResult_COM6[13]       = {"Not Tested"};
char sResult_COM7[13]       = {"Not Tested"};
char sResult_COM8[13]       = {"Not Tested"};
char sResult_DIO1[13]       = {"Not Tested"};
char sResult_DIO2[13]       = {"Not Tested"};
char sResult_LCD[13]        = {"Not Tested"};
char sResult_RS485[13]      = {"Not Tested"};
char sResult_socket[13]     = {"Not Tested"};
char sResult_battery[13]    = {"Not Tested"};
char sResult_AtoD[13]       = {"Not Tested"};
char sResult_DAC_9700[13]   = {"Not Tested"};
char sResult_DAC[13]        = {"Not Tested"};
char sResult_EXTRESET[13]   = {"----------"};
char sResult_9400Bus[13]    = {"Not Tested"};
char sResult_UserChipSelect[13] = {"Not Tested"};

char sOptionsString[80];

union uEthTemp {
	unsigned char uc[2];
	unsigned int ui;
} EthTemp;

unsigned long Ethernet_MAC;

char Ethernet_MAC_cmd[80];
char Ethernet_MAC_str[25];
char temp_str[10];
char *endptr;

unsigned char Debug;

unsigned char show_eeprom_contents = FALSE;

/****************************************************************************
	FUNCTION PROTOTYPES
****************************************************************************/

void main(int argc, char *argv[]);

void Display( void );
int Display_Get_Options( void );

void ErrorHandler (enum ErrorCodes ErrNum);
void SetConsolePort (int port);
void SetLed (unsigned char LedState);
unsigned char SPI_eeprom_functions( enum SPI_functions function, unsigned char ucData );
unsigned char SPI_transfer( unsigned char ucSendData );
int test_5600_analog( void );
int test_atod( void );
int test_9700( void );
void program_MAC_address( void );
int test_Ethernet( void );
int test_RTC( void );
int test_COMx( unsigned char ucComNum );
int test_DIO2_5500( void );
int test_DIO1_5300( void );
int test_DIO2_5300( void );
int test_DIO1_3200( void );
int test_DIO2_3200( void );
int test_LCD( void );
int test_keypad( void );
int test_RS485( unsigned char ucComNum );
int test_socket( unsigned char TestType );
int test_9400Bus (void);
int test_UserChipSelect(void);

/****************************************************************************
	EXTERNAL DECLARATIONS
****************************************************************************/

/****************************************************************************
	Function: main
 Description: display startup info, then display menu and loop until exit
	  Passed: none
	 Returns: none
*****************************************************************************/
void main(int argc, char *argv[])
{
// ----------------------------------------------------------------------------
//	Collect information from BIOS and hardware
// ----------------------------------------------------------------------------

	char *cmd = argv[1];

	Debug = FALSE;

	if( cmd[0] == 'D' )			// display debug messages during test
		Debug = TRUE;

	if( cmd[0] == 'K' )			// enable matrix keypad tests during LCD test
		Options.Keypad = TRUE;

	if( cmd[0] == 'E' )			// used for SPI eeprom debug on TS-5600/5700
		show_eeprom_contents = TRUE;


	HARDWARE_DETECT:
	// read option bits from PLD and set appropriate bits in Options struct

	memset(&Options, 0,sizeof(Options) );  /* Clear all Options */

	gettime (&t);
	getdate (&d);

	// setup register set for call to TS special BIOS function to return model #

	regs.x.ax = 0xB000;					// TS BIOS info function
	int86 (0x15, &regs, &regs);		// Call to INT 15 function xB000

	if( Debug ) {
		printf( "===========================================================\n" );
		printf( "call to INT 15 function xB000 for TS version info, returns:\n\n" );
		printf( "ah=x%02X   (0 if no errors)\n",                 regs.h.ah );
		printf( "al=x%02X   (special version, normal = 0)\n",    regs.h.al );
		printf( "bh=x%02X   (major version #)\n",                regs.h.bh );
		printf( "bl=x%02X   (minor version #)\n",                regs.h.bl );
		printf( "ch=x%02X   (flash size / 512)\n",               regs.h.ch );
		printf( "cl=x%02X   (TS model identifier)\n",            regs.h.cl );
		printf( "dx=x%04X   (BIOS id, default = 'TS')\n",           regs.x.dx );
		printf( "===========================================================\n" );
	}

	ucBiosMajor = regs.h.bh;				// store BIOS major revision number
	ucBiosMinor = regs.h.bl;				// store BIOS minor revision number

	// check for reg ah==0 and dx='TS' this means TS BIOS found

	if( regs.h.ah==0 && regs.x.dx==0x5453 )
		iModel = regs.h.cl;
	else
		ErrorHandler( NON_TS_BIOS );

	// test for development version and display

	if( Debug ) {
		if( regs.h.al == 0xFF )
			printf( "Beta version" );
	}

	// test for special version and display

	if( Debug ) {
		if( regs.h.al > 0 )
			printf( "special version (%d)\n", regs.h.al );
	}

	// make model number string from TS model number id

	switch( iModel ) {
		case TS2100: sprintf( sModelNumber, "TS-2100" ); break;
		case TS2200: sprintf( sModelNumber, "TS-2200" ); break;
		case TS2350: sprintf( sModelNumber, "TS-2350" ); break;
		case TS2800: sprintf( sModelNumber, "TS-2800" ); break;
		case TS3100: sprintf( sModelNumber, "TS-3100" ); break;
		case TS3200: sprintf( sModelNumber, "TS-3200" ); break;
		case TS3300: sprintf( sModelNumber, "TS-3300" ); break;
		case TS3400: sprintf( sModelNumber, "TS-3400" ); break;
		case TS4150: sprintf( sModelNumber, "TS-4150" ); break;
		case TS5300: sprintf( sModelNumber, "TS-5300" ); break;
		case TS5400: sprintf( sModelNumber, "TS-5400" ); break;
		case TS5500: sprintf( sModelNumber, "TS-5500" ); break;
		case TS5600: sprintf( sModelNumber, "TS-5600" ); break;
		case TS5700: sprintf( sModelNumber, "TS-5700" ); break;
	}

	// call BIOS function x12 to get lower ram size

	int86 (0x12, &regs, &regs);
	uiLowRamSize = regs.x.ax;

	// call BIOS function x15 sub-function x88 to get extended memory size

	regs.h.ah = 0x88;
	int86 (0x15, &regs, &regs);
	ulExtRamSize = regs.x.ax;

	if( Debug ) {
		if( regs.x.cflag )
			printf("\nFailure of Int15 func. 88 call\n" );
		if( regs.h.ah == 0x86 )
			printf("\nProtected mode services not available\n" );
	}

	// call BIOS function x15, subfunction xB020 to read jumper block

	regs.x.ax = 0xB020;
	int86 (0x15, &regs, &regs);
	uiJumpers = regs.x.bx;

	if( Debug ) {
		printf( "jumpers=%04X\n", uiJumpers );
	}


	// call BIOS function x15, subfunction B021 to read JP5
	// this is the push switch interrupt call, included for backward compatability

	regs.x.ax = 0xB021;
	int86 (0x15, &regs, &regs);
	//printf ("Push switch settings = (%04X)\n", regs.x.bx );

	//Test for presence of TS-9400
	if( inportb(TS9400BASE+4) == 94 )
		Options.TS9400 = 1;

	switch( iModel ) {
		case TS2350:    break;
		case TS2100: //----------------------------------------------------------
			{
				if( inportb(0x75) & 0x01 )	Options.SRAM = TRUE;
				// determine if 32k, 128k, or 512k
				if( inportb(0x76) & 0x01 )	Options.SRAM = TRUE;
			}
			break;
		case TS2200: //----------------------------------------------------------
			{
				if( inportb(0x75) & 0x01 )	Options.SRAM = TRUE;
				// determine if 32k, 128k, or 512k
				if( inportb(0x76) & 0x01 )	Options.RS485 = TRUE;;
				Options.ETHERNET = TRUE;
			}
			break;
		case TS2800: //----------------------------------------------------------
			{
				if( inportb(0x75) & 0x01 )	Options.SRAM = TRUE;
				// determine if 32k, 128k, or 512k
				if( inportb(0x76) & 0x01 )	Options.RS485 = TRUE;
				if( inportb(0x7D) & 0x01 )	Options.ATOD = TRUE;
				Options.ETHERNET = TRUE;
			}
			break;
		case TS3100: //---------------------------------------------------------
		case TS3200: //----------------------------------------------------------
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;
			}
			break;
		case TS3300: //----------------------------------------------------------
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE ;
				if( inportb( PLD_OPT1_ADDR ) & PLD_ATOD_SET )	Options.ATOD = TRUE;
				Options.ETHERNET = TRUE;
			}
			break;
		case TS3400: //----------------------------------------------------------
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;
				if( inportb( PLD_OPT2_ADDR ) & PLD_EXTRES_SET )	Options.EXTRESET = TRUE;
			}
			break;
		case TS4150: //----------------------------------------------------------
			{
				Options.ETHERNET = TRUE;
				Options.RS485 = TRUE;
			}
			break;
		case TS5300:
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;
				Options.ETHERNET = TRUE;
			}
			break;
		case TS5400:
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;
				Options.ETHERNET = TRUE;
				Options.EXTRESET = TRUE;
				break;
			}
		case TS5500:
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_SRAM_SET )	Options.SRAM = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;
				if( inportb( PLD_OPT1_ADDR ) & PLD_ATOD_SET )	Options.ATOD = TRUE;
				Options.ETHERNET = TRUE;
				Options.EXTRESET = TRUE;
				break;
			}
		case TS5600:
			{

				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;

				ucTemp1 = SPI_eeprom_functions( EEPROM_READ_OPTIONS_BYTE, 0 );

				if( (ucTemp1 & OP_SRAM) != OP_SRAM )	Options.SRAM = TRUE;
				if( (ucTemp1 & OP_EXT) != OP_EXT )		Options.EXT = TRUE;
				if( (ucTemp1 & OP_ADC) != OP_ADC )		Options.ATOD = TRUE;
				if( (ucTemp1 & OP_DAC) != OP_DAC )		Options.DAC = TRUE;
				Options.ETHERNET = TRUE;
				Options.EXTRESET = TRUE;
				break;
			}
		case TS5700:
			{
				if( inportb( PLD_OPT1_ADDR ) & PLD_RS485_SET )	Options.RS485 = TRUE;

				ucTemp1 = SPI_eeprom_functions( EEPROM_READ_OPTIONS_BYTE, 0 );

				if( (ucTemp1 & OP_SRAM) != OP_SRAM )	Options.SRAM = TRUE;
				if( (ucTemp1 & OP_EXT) != OP_EXT )		Options.EXT = TRUE;
				if( (ucTemp1 & OP_ADC) != OP_ADC )		Options.ATOD = TRUE;
				if( (ucTemp1 & OP_DAC) != OP_DAC )		Options.DAC = TRUE;
				Options.ETHERNET = TRUE;
				Options.EXTRESET = TRUE;
				break;
			}
	}

	// check for industrial temp version bit set

	if( inportb( PLD_OPT1_ADDR ) & PLD_IND_SET )	Options.Industrial = TRUE;

	// Write to scratch pad register on COM3 & 4, then read back and compare.

	outportb( COM3BASE+SCRATCH, 0x55 );
	outportb( COM4BASE+SCRATCH, 0xAA );

	outportb( 0x80, 0x00 );		// write to post code to erase bus memory

	ucTemp1 = inportb( COM3BASE+SCRATCH );
	ucTemp2 = inportb( COM4BASE+SCRATCH );

	if( ucTemp1 == 0x55 )
		Options.COM3 = TRUE;
	if( ucTemp2 == 0xAA )
		Options.COM4 = TRUE;

	// Write to scratch pad register on COM5 & 6, then read back and compare.

	outportb( COM5BASE+SCRATCH, 0x55 );
	outportb( COM6BASE+SCRATCH, 0xAA );

	outportb( 0x80, 0x00 );		// write to post code to erase bus memory

	ucTemp1 = inportb( COM5BASE+SCRATCH );
	ucTemp2 = inportb( COM6BASE+SCRATCH );

	if( ucTemp1 == 0x55 )
		Options.COM5 = TRUE;
	if( ucTemp2 == 0xAA )
		Options.COM6 = TRUE;

	// Write to scratch pad register on COM7 & 8, then read back and compare.

	outportb( COM7BASE+SCRATCH, 0x55 );
	outportb( COM8BASE+SCRATCH, 0xAA );

	outportb( 0x80, 0x00 );		// write to post code to erase bus memory

	ucTemp1 = inportb( COM7BASE+SCRATCH );
	ucTemp2 = inportb( COM8BASE+SCRATCH );

	if( ucTemp1 == 0x55 )
		Options.COM7 = TRUE;
	if( ucTemp2 == 0xAA )
		Options.COM8 = TRUE;

	// detect TS-SER4
	ucTemp1 = inportb( TSSER4BASE );
	if( ucTemp1 == 0x7A ) {
		Options.TSSER4 = 1;
		// detect RS-485 option on TS-SER4
		ucTemp1 = inportb( (TSSER4BASE +3) );
		ucTemp2 = ucTemp1 & 0x40;
		if( ucTemp2 == 0x40 ) {
			// if JP1 on TS-SER4 is installed, test COM7 & COM8 as RS-232
			ucTemp2 = ucTemp1 & 0x04;
			if( ucTemp2 != 0x04 ) {
				outportb( (TSSER4BASE+3), 0x80 );
				Options.TSSER4_RS485 = TRUE;
			}
		}
	}

	// detect TS-SER1, can't directly detect, requires some logic
	if( ( Options.COM3 && !Options.COM4 && (iModel != TS5500)) ||
		( Options.COM4 && !Options.COM5 && (iModel == TS5500)) )
		Options.TSSER1 = TRUE;

	// get ethernet MAC from CS8900, assumes packet driver has loaded
	if( (iModel!=TS5400) && (iModel!=TS5500) ) {

		outport( 0x030A, 0x015A );
		EthTemp.ui = inport( 0x030C );

		Ethernet_MAC_str[0] = 0;

		sprintf( temp_str, "00%02X",EthTemp.uc[1] );
		strcat( Ethernet_MAC_str, temp_str );

		outport( 0x030A, 0x015C );
		EthTemp.ui = inport( 0x030C );

		sprintf( temp_str, "%02X%02X", EthTemp.uc[0],EthTemp.uc[1] );
		strcat( Ethernet_MAC_str, temp_str );

		Ethernet_MAC = strtoul( Ethernet_MAC_str, &endptr, 10 );
	}


	// check for bit 0 at 0x77 to be R/W, indicates 128 KB SRAM option

	outportb( 0x77, 0x00 );
	outportb( 0x80, 0x55 );				// erase bus memory
	ucTemp1 = inportb( 0x77 ) & 0x01;

	outportb( 0x77, 0x01 );
	outportb( 0x80, 0x55 );				// erase bus memory
	ucTemp2 = inportb( 0x77 ) & 0x01;

	if( (ucTemp1==0x00) && (ucTemp2==0x01) && (iModel==TS3200) )
		Options.SRAM_128K = TRUE;


	// check for TS-9700 ID byte at 0x161

	ucTemp1 = inportb( TS9700BASE+1 );
	if( ucTemp1 == 0x97 ) {
		Options.TS9700 = TRUE;
		Options.ATOD = TRUE;

		// check for TS-9700 DAC enable bit

		ucTemp1 = inportb( TS9700BASE+6 );
		if( (ucTemp1&0X01) == 0x01 )
			Options.DAC_9700 = TRUE;
	}

// ----------------------------------------------------------------------------
//      MAIN LOOP
// ----------------------------------------------------------------------------

	do {
		Display();

		while( ! kbhit() ) {	// toggle LED at 1/2 Hz just to show we're alive

			gettime (&t);

			if( t.ti_hund > 50 )
				SetLed( ON );
			else
				SetLed( OFF );

		};

		ucKeyPress = toupper( getch() );

		printf( "\n\n" );

		switch( ucKeyPress ) {
			case 'A':
				if( Options.ETHERNET ) {
					if( test_Ethernet() )	sprintf( sResult_Ethernet,  "FAILED" );
					else					sprintf( sResult_Ethernet,  "Passed" );
				}
				if( test_RTC() )			sprintf( sResult_RTC,       "FAILED" );
				else						sprintf( sResult_RTC,       "Passed" );
				if( (!(Options.RS485)) || (iModel == TS5500) ) {
					if( test_COMx(1) )		sprintf( sResult_COM1,      "FAILED" );
					else					sprintf( sResult_COM1,      "Passed" );
				}
				if( test_COMx(2) )			sprintf( sResult_COM2,      "FAILED" );
				else						sprintf( sResult_COM2,      "Passed" );

				if( (iModel == TS5500) && (Options.RS485) ) {

				} else if( Options.COM3 ) {
					if( test_COMx(3) )
						sprintf( sResult_COM3,     "FAILED" );
					else
						sprintf( sResult_COM3,     "Passed" );
				}
				if( Options.COM4 ) {
					if( test_COMx(4) )		sprintf( sResult_COM4, "FAILED");
					else					sprintf( sResult_COM4, "Passed");
				}
				if( Options.COM5 ) {
					if( test_COMx(5) )		sprintf( sResult_COM5,     "FAILED" );
					else					sprintf( sResult_COM5,     "Passed" );
				}
				if( Options.COM6 ) {
					if( test_COMx(6) )		sprintf( sResult_COM6, "FAILED");
					else					sprintf( sResult_COM6, "Passed");
				}
				if( Options.COM7 && (! Options.TSSER4_RS485) ) {
					if( test_COMx(7) )		sprintf( sResult_COM7,     "FAILED" );
					else					sprintf( sResult_COM7,     "Passed" );
				}
				if( Options.COM8 && (! Options.TSSER4_RS485) ) {
					if( test_COMx(8) )		sprintf( sResult_COM8, "FAILED");
					else					sprintf( sResult_COM8, "Passed");
				}
				if( Options.TS9400 ) {
					if( test_9400Bus() )	sprintf( sResult_9400Bus, "FAILED");
					else					sprintf(sResult_9400Bus, "Passed");
					if( test_UserChipSelect() )	sprintf( sResult_UserChipSelect, "FAILED");
					else						sprintf( sResult_UserChipSelect, "Passed");
				}
				if( (iModel == TS3200) ||
					(iModel == TS3100) ||
					(iModel == TS3400) ) {
					if( test_DIO1_3200() )	sprintf( sResult_DIO1, "FAILED" );
					else					sprintf( sResult_DIO1, "Passed" );
				}
				if( (iModel == TS5300) ||
					(iModel == TS5500) ||
					(iModel == TS3300) ) {
					if( test_DIO1_5300() )	sprintf( sResult_DIO1, "FAILED" );
					else					sprintf( sResult_DIO1, "Passed" );
				}
				if( (iModel == TS3200) ||
					(iModel == TS3100) ||
					(iModel == TS3400) ) {
					if( test_DIO2_3200() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5300) ||
					(iModel == TS3300) ) {
					if( test_DIO2_5300() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5400) ||
					(iModel == TS5500) ) {
					if( test_DIO2_5500() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5600) ||
					(iModel == TS5700) ) {
					if( test_DIO_5600() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( Options.ATOD ) {
					if( iModel == TS5600 ) {
						if( test_5600_analog() )	sprintf( sResult_AtoD, "FAILED" );
						else						sprintf( sResult_AtoD, "Passed" );
					} else {
						if( test_atod() )	sprintf( sResult_AtoD, "FAILED" );
						else				sprintf( sResult_AtoD, "Passed" );
					}
				}
				if( test_LCD() )			sprintf( sResult_LCD, "FAILED" );
				else						sprintf( sResult_LCD, "Passed" );
				if( Options.RS485 ) {
					if( iModel == TS5500 ) {
						if( test_RS485(3) )	sprintf( sResult_RS485, "FAILED" );
						else				sprintf( sResult_RS485, "Passed" );
					} else {
						if( test_RS485(1) )	sprintf( sResult_RS485, "FAILED" );
						else				sprintf( sResult_RS485, "Passed" );
					}
				}
				if( test_socket( NORMAL_SRAM) )
					sprintf( sResult_socket,    "FAILED" );
				else						sprintf( sResult_socket, "Passed" );
				break;

			case 'E':
				if( Options.ETHERNET ) {
					if( test_Ethernet() )	sprintf( sResult_Ethernet, "FAILED" );
					else					sprintf( sResult_Ethernet, "Passed" );
				}
				break;
			case '+':
				Debug = ~Debug;
				break;
			case 'C':
				if( test_RTC() )			sprintf( sResult_RTC, "FAILED" );
				else						sprintf( sResult_RTC, "Passed" );
				break;
			case '1':
				if( test_COMx(1) )			sprintf( sResult_COM1, "FAILED" );
				else						sprintf( sResult_COM1, "Passed" );
				break;
			case '2':
				if( test_COMx(2) )			sprintf( sResult_COM2, "FAILED" );
				else						sprintf( sResult_COM2, "Passed" );
				break;
			case '3':
				if( Options.COM3 ) {
					if( test_COMx(3) )		sprintf( sResult_COM3, "FAILED" );
					else					sprintf( sResult_COM3, "Passed" );
				}
				break;
			case '4':
				if( Options.COM4 ) {
					if( test_COMx(4) )		sprintf( sResult_COM4, "FAILED" );
					else					sprintf	(sResult_COM4, "Passed" );
				}
				break;
			case '5':
				if( Options.COM5 ) {
					if( test_COMx(5) )		sprintf( sResult_COM5, "FAILED" );
					else					sprintf( sResult_COM5, "Passed" );
				}
				break;
			case '6':
				if( Options.COM6 ) {
					if( test_COMx(6) )		sprintf( sResult_COM6, "FAILED" );
					else					sprintf	(sResult_COM6, "Passed" );
				}
				break;
			case '7':
				if( Options.COM7 && Options.TSSER4_RS485 ) {
					if( test_RS485(7) )		sprintf( sResult_COM7, "FAILED" );
					else					sprintf( sResult_COM7, "Passed" );
				} else if( Options.COM7 ) {
					if( test_COMx(7) )		sprintf( sResult_COM7, "FAILED" );
					else					sprintf( sResult_COM7, "Passed" );
				}

				break;
			case '8':
				if( Options.COM8 && Options.TSSER4_RS485 ) {
					if( test_RS485(8) )		sprintf( sResult_COM8, "FAILED" );
					else					sprintf( sResult_COM8, "Passed" );
				} else if( Options.COM8 ) {
					if( test_COMx(8) )		sprintf( sResult_COM8, "FAILED" );
					else					sprintf	(sResult_COM8, "Passed" );
				}
				break;
			case 'R':
				if( Options.RS485 ) {
					if( iModel == TS5500 ) {
						if( test_RS485(3) )	sprintf( sResult_RS485,     "FAILED" );
						else				sprintf( sResult_RS485,     "Passed" );
					} else {
						if( test_RS485(1) )	sprintf( sResult_RS485,     "FAILED" );
						else				sprintf( sResult_RS485,     "Passed" );
					}
				}
				break;
			case 'D':
				if( (iModel == TS3200) ||
					(iModel == TS3400) ||
					(iModel == TS3100) ) {
					if( test_DIO1_3200() )	sprintf( sResult_DIO1, "FAILED" );
					else					sprintf( sResult_DIO1, "Passed" );
				}
				if( (iModel == TS5300) ||
					(iModel == TS5500) ||
					(iModel == TS3300) ) {
					if( test_DIO1_5300() )	sprintf( sResult_DIO1, "FAILED" );
					else					sprintf( sResult_DIO1, "Passed" );
				}
				break;

			case 'T':
				if( (iModel == TS3200) ||
					(iModel == TS3400) ||
					(iModel == TS3100) ) {
					if( test_DIO2_3200() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5300) ||
					(iModel == TS3300) ) {
					if( test_DIO2_5300() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5400) ||
					(iModel == TS5500) ) {
					if( test_DIO2_5500() )	sprintf( sResult_DIO2, "FAILED" );
					else					sprintf( sResult_DIO2, "Passed" );
				}
				if( (iModel == TS5600) ||
					(iModel == TS5700) ) {
					if( test_DIO_5600() )	sprintf( sResult_DIO2,      "FAILED" );
					else					sprintf( sResult_DIO2,      "Passed" );
				}
				break;
			case 'N':
				Options.EXTRESET = FALSE;
				test_DIO2_3200();
				break;
			case 'L':
				if( test_LCD() )			sprintf( sResult_LCD, "FAILED" );
				else						sprintf( sResult_LCD, "Passed" );
				break;
			case 'S':
				if( test_socket( NORMAL_SRAM ) )
					sprintf( sResult_socket, "FAILED" );
				else						sprintf( sResult_socket, "Passed" );
				break;
			case 'B':
				if( test_socket( BATTERY_SRAM ) )
					sprintf( sResult_battery, "FAILED" );
				else						sprintf( sResult_battery, "Passed" );
				break;
			case 'O':
				if( Options.ATOD ) {
					if( iModel == TS5600 ) {
						if( test_5600_analog() )	sprintf( sResult_AtoD, "FAILED" );
						else						sprintf( sResult_AtoD, "Passed" );
					} else {
						if( test_atod() )	sprintf( sResult_AtoD, "FAILED" );
						else				sprintf( sResult_AtoD, "Passed" );
					}
				}
				break;
			case 'V':
				if( Options.DAC_9700 ) {
					if( test_9700() )		sprintf( sResult_DAC_9700, "FAILED" );
					else					sprintf( sResult_DAC_9700, "Passed" );
				}
				break;
			case 'P':
				if( (Options.ETHERNET) && ! ((iModel==TS5400) || (iModel==TS5500)) )
					program_MAC_address();
				break;
			case '9':
				if( Options.TS9400 )
					sprintf( sResult_9400Bus, test_9400Bus() ? "FAILED" : "Passed" );
				break;
			case 'U':
				if( Options.TS9400 )
					sprintf( sResult_UserChipSelect, test_UserChipSelect() ? "FAILED" : "Passed");
				break;
			case CTRL_O:
				Display_Get_Options();
//				if( Display_Get_Options() )		// if options changed re-detect
//					goto HARDWARE_DETECT;		// hardware
				break;
			case 'X':
				exit(0);
				break;
			default:
				printf("\a Invalid key");
		}
		printf("Hit any key...");
		getch();
		printf("\n");
	}
	while( ucKeyPress != 'X' );

/*
				case 'K':	test_keypad();			break;
				case 'Z':	enter_debugger();		break;
*/
}

/****************************************************************************
	Function: Display 
 Description: Draws main screen
	  Passed: none
	 Returns: none
*****************************************************************************/
void Display( void )
{

	system("cls");

	gotoxy( 0,0 );

	printf( "Ŀ");

	printf( "%-19s  Time: %2d:%02d Date: %02d/%02d/%04d  %25s",
			VERSION_STR, t.ti_hour, t.ti_min, d.da_mon, d.da_day, d.da_year, TS_STR );
	printf( "Ĵ");

	printf( " Model Number: %7s  BIOS version: %d.%02d  Base RAM: %-3d  Ext RAM: %-6ld ",
			sModelNumber, ucBiosMajor, ucBiosMinor, uiLowRamSize, ulExtRamSize );

	printf( "Ĵ");


	// the options string is overloaded, nuke some stuff now, fix later. mtj 20021122

	sprintf( sOptionsString, " Options detected: " );

	if( Options.TSSER4_RS485 )	   strcat( sOptionsString, "TS-SER4-RS485 " );
	else if( Options.TSSER4 )	   strcat( sOptionsString, "TS-SER4 " );
	if( Options.RS485 )			   strcat( sOptionsString, "RS485 " );
	if( Options.SRAM_128K )		   strcat( sOptionsString, "128K SRAM " );
	else if( Options.SRAM )		   strcat( sOptionsString, "SRAM " );
	if( Options.EXTRESET )		   strcat( sOptionsString, "Reset " );
	if( Options.Bus16bit )		   strcat ( sOptionsString, "16bit ");
	if( Options.Industrial )	   strcat ( sOptionsString, "Ind ");

	// make this work with the davicom chips, just skip for now... mtj 20021122

	if( (Options.ETHERNET) && !(
							   (iModel==TS5400) || 
							   (iModel==TS5500) || 
							   (iModel==TS5600) || 
							   (iModel==TS5700)) ) {
		if( Ethernet_MAC == 0 || Ethernet_MAC == 123456 )
			// for Crystal CS8900 ethernet adapter, some 3000 series products
			sprintf (Ethernet_MAC_str, "Ethernet: ERR- BAD MAC___ ");
		else
			// for Davicom 9102A ethernet adapter, all 5000 series products
			sprintf( Ethernet_MAC_str, "Ethernet (00D069:%06lu) ", Ethernet_MAC );
		strcat( sOptionsString, Ethernet_MAC_str );
	}

	strcat( sOptionsString, "JP" );
	if( uiJumpers & 0x01 ) strcat( sOptionsString, "1" );
	if( uiJumpers & 0x02 ) strcat( sOptionsString, "2" );
	if( uiJumpers & 0x04 ) strcat( sOptionsString, "3" );
	if( uiJumpers & 0x08 ) strcat( sOptionsString, "4" );
	if( uiJumpers & 0x10 ) strcat( sOptionsString, "5" );
	if( uiJumpers & 0x20 ) strcat( sOptionsString, "6" );
	if( uiJumpers & 0x80 ) strcat( sOptionsString, "8" );
	strcat( sOptionsString, " " );

	if( Options.TS9400 )	strcat (sOptionsString, "9400 ");
	if( Options.TS9700 )	strcat( sOptionsString, "9700 " );
	if( Options.ATOD )		strcat( sOptionsString, "A/D " );
	if( Options.DAC )		strcat( sOptionsString, "DAC " );
	if( Options.DAC_9700 )	strcat( sOptionsString, "DAC_9700 " );
	if( Options.EXT )		strcat( sOptionsString, "Ext Temp " );

	printf( "%-78s ", sOptionsString );

	printf( "Ĵ");

//	printf( "%78s", " " );
//	printf( " %7s %-35s %15s %16s ", " ","+   to Enable Diagnostic Mode",     Debug ? "debug ON" : "debug OFF" ," " );
	printf( " %-35s %38s ", "Press   A   to run ALL standard tests", " " );
	if( Options.ETHERNET )
		printf( " %7s %-35s %15s %16s ", " ","E   to Test ethernet:",         sResult_Ethernet,   " " );
	printf( " %7s %-35s %15s %16s ", " ","C   to Test Real Time Clock:",      sResult_RTC,        " " );
	printf( " %7s %-35s %15s %16s ", " ","1   to Test COM1:", sResult_COM1, " " );
	printf( " %7s %-35s %15s %16s ", " ","2   to Test COM2:", sResult_COM2, " " );
	if( Options.COM3 )
		printf( " %7s %-35s %15s %16s ", " ","3   to Test COM3 :", sResult_COM3," " );
	if( Options.COM4 )
		printf( " %7s %-35s %15s %16s ", " ","4   to Test COM4 :", sResult_COM4, " " );
	if( Options.COM5 )
		printf( " %7s %-35s %15s %16s ", " ","5   to Test COM5 :", sResult_COM5," " );
	if( Options.COM6 )
		printf( " %7s %-35s %15s %16s ", " ","6   to Test COM6 :", sResult_COM6, " " );
	if( Options.COM7 ) {
		if( Options.TSSER4_RS485 )
			printf( " %7s %-35s %15s %16s ", " ","7   to Test RS-485 on COM7 :", sResult_COM7," " );
		else
			printf( " %7s %-35s %15s %16s ", " ","7   to Test COM7 :", sResult_COM7," " );
	}
	if( Options.COM8 ) {
		if( Options.TSSER4_RS485 )
			printf( " %7s %-35s %15s %16s ", " ","8   to Test RS-485 on COM8 :", sResult_COM8, " " );
		else
			printf( " %7s %-35s %15s %16s ", " ","8   to Test COM8 :", sResult_COM8, " " );
	}

	if( (iModel != TS5400) && (iModel != TS5600) && (iModel != TS5700) )
		printf( " %7s %-35s %15s %16s ", " ","D   to Test Digital I/O 1:",    sResult_DIO1,       " " );
	printf( " %7s %-35s %15s %16s ", " ","T   to Test Digital I/O 2:",        sResult_DIO2,       " " );
	if( (Options.EXTRESET) && (iModel==TS3400) )
		printf( " %7s %-35s %15s %16s ", " ","N   to Test Reset Function:",    sResult_EXTRESET,       " " );
	//T invokes DIO2 test, which will reset 3400 with OPTION_EXTRESET

	printf( " %7s %-35s %15s %16s ", " ","L   to Test LCD:",                  sResult_LCD,        " " );
	if( Options.RS485 )
		if( iModel != TS5500 )
			printf( " %7s %-35s %15s %16s ", " ","R   to Test RS-485 on Com 1:",  sResult_RS485,      " " );
		else
			printf( " %7s %-35s %15s %16s ", " ","R   to Test RS-485 on Com 3:",  sResult_RS485,      " " );
	if( Options.SRAM )
		printf( " %7s %-35s %15s %16s ", " ","S   to Test SRAM socket:",          sResult_socket,     " " );
	if( Options.ATOD )
		printf( " %7s %-35s %15s %16s ", " ","O   to Test A/D converter:",    sResult_AtoD,       " " );
	if( Options.DAC_9700 )
		printf( " %7s %-35s %15s %16s ", " ","V   to Test analog output:",    sResult_DAC_9700,       " " );
	if( ( Options.SRAM ) || (Options.SRAM_128K) )
		printf( " %7s %-35s %15s %16s ", " ","B   to Test Battery backed SRAM:",sResult_battery,      " " );
	if( Options.TS9400 ) {
		printf( " %7s %-35s %15s %16s ", " ","9   to Test 16-bit Bus :", sResult_9400Bus, " " );
		printf( " %7s %-35s %15s %16s ", " ","U   to Test User Chip Select :", sResult_UserChipSelect, " " );
	}
//	if( (Options.ETHERNET) && ! ((iModel==TS5400) || (iModel==TS5500)) )
//		printf( " %7s %-35s %32s ", " ","P   to Program MAC address", " " );


//	printf( " \tZ\t to Enter special functions menu\n" );
	printf( " %7s %-68s ", " ", "X   to Exit ( '+' to view debug )" );
//	printf( "%78s", " " );
	printf( "");

	printf( "  Enter selection:" );

/*

  special functions
  
	 printf( "\tK\t to Test Keypad: \t\t%s\n", sResult );
	 printf( "\tZ\t to Enter debugger ('g' to exit debug)\n" );
*/

}

/****************************************************************************
	Function: Display_Get_Options
 Description: Draws options screen so user can configure 5600/5700 hardware
	  Passed: none
	 Returns: 0 if no change to options, 1 if changed
*****************************************************************************/
int Display_Get_Options( void )
{

	unsigned char ucOptions = 0;
	unsigned char ucDone = FALSE;
	unsigned char ucReturn = 0;		// assume no changes
	unsigned char ucKeyPress;

	SPI_eeprom_functions( EEPROM_SETUP_MMCR, 0 );

	if( show_eeprom_contents )
		SPI_eeprom_functions( EEPROM_DUMP_DATA, 0 );

	// read eeprom

	ucOptions = SPI_eeprom_functions( EEPROM_READ_OPTIONS_BYTE, 0 );

	// display options and prompt user

	while( ! ucDone ) {

		system("cls");

		gotoxy( 0,0 );

		printf( "\n\n" );
		printf( "\tKey\tOption Name\tEnabled/Disabled" );
		printf( "\n\n" );
		printf( "\t1\tOP-SRAM-32K-NV:    " );
		if( (ucOptions & OP_SRAM ) == OP_SRAM )		printf( "Disabled\n" );
		else										printf( "Enabled\n" );
		printf( "\t2\tOP-ADC:            " );
		if( (ucOptions & OP_ADC ) == OP_ADC )		printf( "Disabled\n" );
		else										printf( "Enabled\n" );
		printf( "\t3\tOP-DAC-2CH:        " );
		if( (ucOptions & OP_DAC ) == OP_DAC )		printf( "Disabled\n" );
		else										printf( "Enabled\n" );
		printf( "\t4\tOP-EXT-TEMP:       " );
		if( (ucOptions & OP_EXT ) == OP_EXT )		printf( "Disabled\n" );
		else										printf( "Enabled\n" );
		printf( "\n\n\tEnter option number to change: ");
		printf( "(ENTER=save, ESC=abort)" );

		// prompt user

		ucKeyPress = getch();

		// process keypress

		switch( ucKeyPress ) {
			case '1':
				ucOptions ^= OP_SRAM;
				break;
			case '2':
				ucOptions ^= OP_ADC;
				break;
			case '3':
				ucOptions ^= OP_DAC;
				break;
			case '4':
				ucOptions ^= OP_EXT;
				break;
			case ESC:
				printf( "\n\nAborting with no changes..." );
				ucDone = TRUE;
				break;
			case CR:
				printf( "\n\nSaving to eeprom..." );
				SPI_eeprom_functions( EEPROM_WRITE_STATUS_REG, 0x00 );
				SPI_eeprom_functions( EEPROM_CHIP_WE, 0 );
				SPI_eeprom_functions( EEPROM_WRITE_OPTIONS_BYTE, ucOptions );
				ucDone = TRUE;
				ucReturn = 1;
				break;
		}

	}

	// save to eeprom
	return( ucReturn );
}
/****************************************************************************
	Function: ErrorHandler
 Description: Print error msg to standard output, then exit
	  Passed: error code
	 Returns: none
*****************************************************************************/
void ErrorHandler (enum ErrorCodes ErrNum)
{
	printf( "\aError (%s) exiting program\n", ErrorMesg[ErrNum] );

	exit(ErrNum);
}

/****************************************************************************
	Function: SetConsolePort
 Description: Use special General Software BIOS calls to change the console 
	  Passed: Port to use for console: 0=video/none, 1=COM1, 2=COM2, ...
	 Returns: none
*****************************************************************************/
void SetConsolePort (int port)
{
	union REGS regs;

	regs.h.ah = 0xA1;				// Set console redirection function
	regs.x.bx = port;				// Set which port
	int86 (0x15, &regs, &regs);		// do it.
}

/****************************************************************************
	Function: SetLed
 Description: Use special TS BIOS call to control the status LED.
	  Passed: LED state, 0=LED off, 1=LED on
	 Returns: none
*****************************************************************************/
// Make the BIOS call to change the LED
void SetLed (unsigned char LedState)
{
	union REGS regs;

	if( LedState ) {
		regs.h.bh = 0x01;			// Turn LED on
//		outportb(0xf862,(0xBF&inportb(0xf862)));	// works on 386
	} else {
		regs.h.bh = 0x00;			// Turn LED off
//		outportb(0xf862,(0x40|inportb(0xf862)));	// works on 386
	}

	regs.x.ax = 0xB010;				// Set LED BIOS call

	int86 (0x15, &regs, &regs);		// do it.
}

/****************************************************************************
	Function: test_5600_analog
 Description: Use DAC or fixture to provide test signal for A/D on TS-56/5700
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure
*****************************************************************************/
int test_5600_analog( void )
{
	// an external test fixture provides fixed voltages for each analog input
	// 0,2,4 are set to 2.35 VDC, 1,3,5 are set to 1.18 VDC, if a DAC is
	// present set A to 2.35v and B to 1.18v and loop back to analog inputs

	unsigned char error = FALSE;

//	unsigned char i,j;

	unsigned int uiTemp;
	unsigned char ucTemp;

	unsigned char ucAtoDSend = 0;

	unsigned char ucAtoDChan = 0;

	unsigned int uiAtoDData = 0;

	unsigned char ucDoTwice = 1;

	float result;

	/*
	on TS-5600 and TS-5700:
	PIO4-7 are user CS0-3
	PIO15 is DAC CS
	PIO27 is A/D CS
	EEPROM Cs is 0x76, bit 6

	*/

	int MMCR_segment = 0xDF00;
	unsigned char far *char_loc;
	unsigned int far *int_loc;

	printf( "Testing Analog ");

	// set PIO data registers to 1's =========================================

	// set PIO 4-7,15 to one
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_SET );
	uiTemp = *int_loc;
	*int_loc = 0x80F0;

	// set PIO 27 to one
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_SET );
	uiTemp = *int_loc;
	*int_loc = 0x0800;


	// set PIO direction registers to outputs ================================

	// set PIO 4-7,15 Direction to outputs
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_DIR );
	uiTemp = *int_loc;
	*int_loc |= 0x80F0;

	// set PIO 27 Direction to output
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_DIR );
	uiTemp = *int_loc;
	*int_loc |= 0x0800;


	// set PIO PFS registers to PIO mode =====================================

	// set PIO 4-7,15 Pin Function Select to PIO mode
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_PFS );
	uiTemp = *int_loc;
	*int_loc &= 0x7F0F;

	// set PIO 27 Pin Function Select to PIO mode
	int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_PFS );
	uiTemp = *int_loc;
	*int_loc &= 0xF7FF;

	if( Debug )
		printf( "press a key to exit..." );

	if( Debug ) {
		printf( "\nA/D:	0	1	2	3	4	5\n" );
	}

	do {

		if( Debug )
			printf("\r     ");

		if( Options.DAC ) {

			//---------------------------------------------------------------------
			// Write to DAC B buffer, set for 1.12v, same as test fixture
			//---------------------------------------------------------------------

			// enable DAC CS on PIO15
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_CLEAR );
			*int_loc = 0x8000;

			uiTemp = 0x0785;

			uiTemp |= 0x4000;

			ucTemp = uiTemp >> 0x08;
			SPI_transfer( ucTemp );

			ucTemp = uiTemp;
			SPI_transfer( ucTemp );

			// disable DAC CS on PIO15
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_SET );
			*int_loc = 0x8000;

			//--------------------------------------------------------------------
			// Write to DAC A and update DAC A and DAC B, A set to 2.35v
			//--------------------------------------------------------------------

			// setup control register for DAC
			char_loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_CTRL_REG );
			*char_loc = 0x03;

			// enable DAC CS on PIO15
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_CLEAR );
			*int_loc = 0x8000;

			uiTemp = 0x039E;

			uiTemp |= 0x8000;

			ucTemp = uiTemp >> 0x08;
			SPI_transfer( ucTemp );

			ucTemp = uiTemp;
			SPI_transfer( ucTemp );

			// disable DAC CS on PIO15
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_SET );
			*int_loc = 0x8000;

			delay(1);
		}
		//--------------------------------------------------------------------

		for( ucAtoDChan=0; ucAtoDChan<6; ucAtoDChan++ ) {

			// setup control register for A/D
			char_loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_CTRL_REG );
			*char_loc = 0x21;

			// enable A/D CS on PIO27
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_CLEAR );
			*int_loc = 0x0800;

			ucAtoDSend = (0x04 | ( ucAtoDChan<<3 ));

			// start analog conversion
			ucTemp = SPI_transfer( ucAtoDSend  );
			ucTemp = SPI_transfer( 0x04 );

			// disable A/D CS on PIO27
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_SET );
			*int_loc = 0x0800;

			delay(1);

			// enable A/D CS on PIO27
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_CLEAR );
			*int_loc = 0x0800;

			// get analog conversion
			ucTemp = SPI_transfer( ucAtoDSend  );
			uiAtoDData = ucTemp*0xFF;
			ucTemp = SPI_transfer( 0x04 );
			uiAtoDData += ucTemp;

			// disable A/D CS on PIO27
			int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_SET );
			*int_loc = 0x0800;

			// calculate result as a percent of full scale
			result = 100.0 * ((float) uiAtoDData) / 4096.0;

			if( Debug )
				printf("(%3.1f)  ", result );

			// almost every channel is a special case, so this is easier
			switch( ucAtoDChan ) {
				case 0: if( (result<20) || (result>27) ) error = TRUE;break;
				case 1: if( (result<8 ) || (result>15) ) error = TRUE;break;
				case 2: if( (result<20) || (result>27) ) error = TRUE;break;
				case 3: if( (result<41) || (result>48) ) error = TRUE;break;
				case 4: if( (result<91) || (result>98) ) error = TRUE;break;
				case 5: if( (result<41) || (result>48) ) error = TRUE;break;
			}
		}

		if( ucDoTwice )	error = FALSE;	// ignore errors on first pass, DACs not OK

	} while( (! kbhit() && Debug) || ucDoTwice-- );


	if( ! error ) {
		printf( "OK\n" );
		return 0;
	} else {
		printf( "\a Failed" );
		return 1;
	}

}

/****************************************************************************
	Function: SPI_eeprom_functions
 Description: Send write enable command to SPI eeprom on TS-5600/TS-5700
	  Passed: enun is SPI eeprom function to execute and optional data
	 Returns: data read in from eeprom
*****************************************************************************/
unsigned char SPI_eeprom_functions( enum SPI_functions function, unsigned char ucData )
{

	int j;
	unsigned char far *char_loc;
	int MMCR_segment = 0xDF00;

	unsigned char ucReturnData = 0;

	switch( function ) {
		
		case EEPROM_CHIP_WE:	//--------------------------------------------

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send write enable command
			SPI_transfer( 0x06 );

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			delay( 1 );

			break;

		case EEPROM_WRITE_STATUS_REG:	//------------------------------------

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send write status register command
			SPI_transfer( 0x01 );

			// send data to status register
			SPI_transfer( ucData );

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			delay( 1 );

			break;

		case EEPROM_WRITE_OPTIONS_BYTE:	//------------------------------------

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send write data command
			SPI_transfer( 0x02 );

			// send address of options byte 
			SPI_transfer( 0x80 );

			// send data
			SPI_transfer( ucData );

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			delay( 1 );


			break;

		case EEPROM_READ_OPTIONS_BYTE:	//------------------------------------

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send read data command
			SPI_transfer( 0x03 );

			// send address of options byte 
			SPI_transfer( 0x80 );

			// send get data
			ucReturnData = SPI_transfer( 0xFF );

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			delay( 1 );

			break;

		case EEPROM_DUMP_DATA:

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send read command
			SPI_transfer( 0x03 );

			// send address 0x00
			SPI_transfer( 0x00 );

			// read entire 256 bytes
			for( i=0; i<16; i++ ) {
				for( j=0; j<16; j++ ) {

					ucTemp1 = SPI_transfer( 0xFF );

					printf(" %02X", ucTemp1);
				}
				printf("\n");
			}

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			getch();

			break;

		case EEPROM_SETUP_MMCR:
			// setup control register
			char_loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_CTRL_REG );
			*char_loc = 0x37;

			break;
	}

	return( ucReturnData );

}

// misc SPI code used during development for TS-5600 and TS-5700

/*
		// set PIO 4-7,15 outputs
		int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_SET );
		*int_loc = 0x80F0;

		// set PIO 27 output
		int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_SET );
		*int_loc = 0x0800;

		// eeprom CS on
		ucTemp1 = inportb( 0x76 );
		outportb( 0x76, ucTemp1 | 0x40 );

		delay( 50 );

		// clear PIO 4-7 outputs
		int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_0_15_CLEAR );
		*int_loc = 0x00F0;

		// clear PIO 27 output
		int_loc = (unsigned int far *) MK_FP ( MMCR_segment, PIO_16_31_CLEAR );
		*int_loc = 0x0800;

		// eeprom CS off
		ucTemp1 = inportb( 0x76 );
		outportb( 0x76, ucTemp1 & 0xBF );

		delay( 50 );

		// write some test data to eeprom ========================================

		for( i=0; i<16; i++ ) {

			// must write enable each page
			SPI_eeprom_write_enable();

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// send write data command
			SPI_transfer( 0x02 );

			// send address
			SPI_transfer( (unsigned char) (i*16) );

			for( j=0; j<16; j++ ) {

				// send data
				SPI_transfer( (unsigned char) (i*16+j) );
			}

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			delay(1);

			// eeprom CS on
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 | 0x40 );

			// read status reg to check busy bit
			SPI_transfer( 0x05 );

			for( j=0; j<100; j++ ) {

				ucTemp1 = SPI_transfer( 0xFF );

				// done if clear
				if( (ucTemp1 & 0x01) == 0x00 )
					break;

				delay( 1 );
			}

			// eeprom CS off
			ucTemp1 = inportb( 0x76 );
			outportb( 0x76, ucTemp1 & 0xBF );

			if( j>99 )
				printf( "Time out waiting for busy bit to clear" );
		}
*/
/****************************************************************************
	Function: SPI_transfer
 Description: Use SPI functions on AMD Elan processor to transfer data
	  Passed: Data to send to SPI device
	 Returns: Data returned from SPI device
*****************************************************************************/
unsigned char SPI_transfer( unsigned char ucSendData )
{

	unsigned char ucStatus;
	int MMCR_segment = 0xDF00;
	unsigned char far *loc;


	// setup transmit data
	loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_TX_DATA );
	*loc = ucSendData;

	// start transaction
	loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_CMD_REG );
	*loc = 0x03;

	// wait for busy bit to clear
	do {
		loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_STATUS_REG );
		ucStatus = *loc;
	} while( (ucStatus & 0x02) != 0x00 );

	// read data
	loc = (unsigned char far *) MK_FP ( MMCR_segment, SPI_RX_DATA );

	return *loc;
}

/****************************************************************************
	Function: test_atod
 Description: Use external test fixture to perform go/no go test on analog input
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure
*****************************************************************************/
int test_atod( void )
{

	// an external test fixture provides fixed voltages for each analog input
	// 0,2,4,8 are set to 2.35 VDC, 1,3,5,7 are set to 1.18 VDC

	union REGS regs;
	float result;
	int i,j;
	unsigned int iWord;
	unsigned char error = FALSE;

	printf( "Testing analog input : " );

	if( Debug )
		printf( "\nAtoD result Ch   0     1     2     3     4     5     6     7 (any key to exit)\n" );

	do {

		if( Debug )
			printf( "               " );

		for( i=0; i<8; i++ ) {

			if( Options.TS9700 == FALSE ) {		// if not a TS-9700 this must be analog input on
												// the base board. Only available on 3300 and 5500
				regs.x.ax = 0xB050;				// A/D conversion BIOS function
				regs.h.bl = 0x10+i;				// 0-10v, unipolar, channel i
				int86 (0x15, &regs, &regs);		// call BIOS function
				if( regs.h.ah != 0 ) {
					printf( "\a Failed - BIOS call returned error (%d)\n", regs.h.ah );
					return 1;
				}
				result = (((float) regs.x.bx)/4096.0) * 100.0;

			} else {	// TS-9700 test here

				// start atod conversion
				ucTemp1 = 0x20 + i;
				outportb( TS9700BASE, ucTemp1 );

				// wait until complete
				do {
					ucTemp1 = inportb(TS9700BASE) & 0x80;

				} while( ( ucTemp1 ) == 0 );

				// get atod result
				iWord = inport( TS9700BASE+2 );

				//convert to percent
				result = (((float) iWord)/4096.0) * 100.0;
			}

			if( Debug )
				printf( "%3.1f\% ", result );

			// the test fixture connects 4 inputs to 2.35 VDC and 4 to 1.18 VDC

			if( (i&0x01) == 0x00 ) {	// even inputs are set to 2.35 VDc

				if( (result < 91) || (result > 97 ) ) {	 //nominal is 94%
					if( Debug )
						printf( "ch %d result = %3.1f (good = 91%-97%)\n", i, result );
					error = TRUE;
				}
			} else {					// odd inputs are set to 1.18 VDC

				if( (result < 44) || (result > 50 ) ) {	 //nominal is 47%
					if( Debug )
						printf( "ch %d result = %3.1f (good = 47%-50%)\n", i, result );
					error = TRUE;
				}
			}
		}

		if( Debug )
			printf("\r");

	} while( ! kbhit() && Debug );

	if( ! error ) {
		printf( "OK\n" );
		return 0;
	} else {
		printf( "\a Failed" );
		return 1;
	}
}
// Notes copied from BIOS code about using function xB050

//	BL = 0 0 0 R B C2 C1 C0 where
//		R - Range	0 = 5V;		1 = 10V
//		B - Bipolar Sel	0 = unipolar;	1 = bipolar
//		C0-C2 - channel # 0 to 7
//
//   Output:
//	AH = 	0 - No Error - results in BX
//		1 - bad subfunction
//		2 - bad input registers (maybe if BX.5 - 7 set?)
//		3 - ADC option not present (MACH 07Dh.0 = 0)
//		4 - HW error - ADC not present or timed-out
//	BX = 	results
//			if unipolar mode, BX.4-7 = 0
//			if bipolar mode, BX.4-7 = sign extension
//			(handled auto. by the ADC itself)
//	CY	- set if error, else clear if success.

// this code will work directly with hardware.
// active low conv. conplete signal connected to port 1 pin 5 on TS-3300

//	outportb( 0x196,0x00 );					// start a conversion
//	while( (inportb(P1PIN) & 0x20) == 0 );	// wait for conversion complete 
//	printf( "x196=%04X\n", inport(0x196) );	// print results

/****************************************************************************
	Function: test_9700
 Description: Use analog input to test analog output, requires loopback cable
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure
*****************************************************************************/
int test_9700( void )
{

	// connect DAC1 to A/D1, DAC2 to A/D2, etc.

	unsigned int uiAnalogIn;
	unsigned int uiAnalogOut;
	unsigned int i,j;
	int result;
	unsigned char error = FALSE;

	printf( "Testing analog output : " );

	for( i=0; i<4; i++ ) {
		for( j=0; j<2048; j=j+250 ) {

			uiAnalogOut = (i<<14) + j;

			// set DAC value
			outport( TS9700BASE+4, uiAnalogOut );

			delay( 25 );

			// start atod conversion
			ucTemp1 = 0x20 + i;
			outportb( TS9700BASE, ucTemp1 );

			// wait until complete
			do {
				ucTemp1 = inportb(TS9700BASE) & 0x80;

			} while( ( ucTemp1 ) == 0 );

			// get atod result
			uiAnalogIn = inport( TS9700BASE+2 );

			result = (uiAnalogIn/2) - j;

			if( (result < -7) || (result > 7 ) ) { //+- 7 bits accuracy
				if( Debug )
					printf( "ch %d result = %d (good = +-7)\n", i, result );
				error = TRUE;
			}

			if( Debug )
				printf( "ch %d DAC = %04X A/D = %04X \n", i, j, uiAnalogIn );

		}
	}

	if( ! error ) {
		printf( "OK\n" );
		return 0;
	} else {
		printf( "\a Failed" );
		return 1;
	}
}

/****************************************************************************
	Function: program_MAC_address
 Description: prompts for S/N then generates the MAC which is stored in eeprom
	  Passed: none
	 Returns: none
*****************************************************************************/
void program_MAC_address( void )
{
	printf( "Enter serial number: " );
	scanf( "%lu", &Ethernet_MAC );
	printf("(%lu)", Ethernet_MAC );
	// generate DOS command line for programming MAC into eeprom
	// this requires an external program and should only be done at the factory
	sprintf( Ethernet_MAC_cmd, "tseeprom ts8900.dat /p 00D069%06lu 00%06lu", Ethernet_MAC, Ethernet_MAC );
	system( Ethernet_MAC_cmd );
}

/****************************************************************************
	Function: test_Ethernet
 Description: Call external ping routing which uses the packet driver
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure 
*****************************************************************************/
int test_Ethernet( void )
{
	unsigned char ping_result;

	printf( "Testing Ethernet: " );

	// the customized ping routine will run 100 pings and return the number of
	// successful pings in the UART scratch pad register.

	system( "ping /tsepc 192.168.0.200 100" );

	ping_result = inportb( 0x2FF );	// get ping result from UART scratch reg.

	if( ping_result == 0 ) {
		printf( "\aError - no response\n" );
		return 1;
	} else if( ping_result < 98 ) {
		printf( "\aError - lost more that 3 packets\n" );
		return 1;
	} else if( ping_result > 127 ) {
		printf( "\aError - Ethernet failed\n" );
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}
}

/****************************************************************************
	Function: test_RTC
 Description: test CMOS memory and check battery good bit where applicable
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure 
*****************************************************************************/
int test_RTC( void )
{
	int y = 0;
	unsigned char ucResult = 0;
	unsigned char ucRTCsave;
	unsigned char ucRTCsave2;

	printf( "Testing RTC : " );

	outportb( RTC_ADDR, 125 );
	ucRTCsave = inportb( RTC_DATA );

	outportb( RTC_ADDR, 126 );
	ucRTCsave2 = inportb( RTC_DATA );

	for( i=0; i<255; i++ ) {
		outportb( RTC_ADDR, 125 );
		outportb( RTC_DATA, i );
		outportb( RTC_ADDR, 126 );
		outportb( RTC_DATA, 12 );
		outportb( RTC_ADDR, 125 );
		if( i != inportb( RTC_DATA) )
			y++;
	}

	outportb( RTC_ADDR, 125 );
	outportb( RTC_DATA, ucRTCsave );
	outportb( RTC_ADDR, 126 );
	outportb( RTC_DATA, ucRTCsave2 );


	// Check the CMOS/RTC battery bit if applicable.

	// The TS-5600 and TS-5700 use the RTC/CMOS inside the AMD Elan processor
	// so the battery bit is always bad unless the TS-5620 external RTC/CMOS
	// board is installed. The custom Technologic Systems diagnostic byte
	// located in CMOS at index 0x50 will have bit 7 set when a 5620 is detected.

	outportb( RTC_ADDR, 0x50 );
	ucTemp1 = inportb( RTC_DATA );		// get TS diagnostic byte

	if( (ucTemp1 & 0x80) == 0x80 ) {	// is bit 7 set?

		if( Debug )
			printf(" Testing battery good bit\n" );

		outportb( RTC_ADDR, 0x0D );		// Yes, check battery good bit

		// bit 7 of address 0x0D is 1 if battery is good, zero if bad
		if( inportb( RTC_DATA ) != 0x80 ) {
			printf( "\aRTC Error: CMOS battery bad\n" );
			ucResult = 1;
		} else {
			if( Debug )
				printf(" Battery backed RTC/CMOS not detected, skip test\n" );
		}
	}

	if( y ) {
		printf( "\aRTC Error: Data write/read not equal.\n" );
		ucResult = 1;;
	}

	return( ucResult );
}

/****************************************************************************
   Function: test_COMx
Description: tests Tx to RX & RTS to CTS loopback connection (special cables)
	 Passed: com port number to test
	Returns: 1 if errors, 0 if no errors
*****************************************************************************/
int test_COMx( unsigned char ucComNum )
{
	unsigned int uiComBase;
	unsigned char ucWaitCount;
	unsigned char ucTestData[2] = { 0x7A, 0x25};

	printf( "Testing COM%d : ", ucComNum );

	// -------------------------------------------------------------------------
	// Setup com port base address
	// -------------------------------------------------------------------------

	switch( ucComNum ) {
		case 1: uiComBase = COM1BASE;   break;
		case 2: uiComBase = COM2BASE;   break;
		case 3: uiComBase = COM3BASE;   break;
		case 4: uiComBase = COM4BASE;   break;
		case 5: uiComBase = COM5BASE;   break;
		case 6: uiComBase = COM6BASE;   break;
		case 7: uiComBase = COM7BASE;   break;
		case 8: uiComBase = COM8BASE;   break;
		default:
			printf("\aCom%d Test Error: Invalid port number\n", ucComNum );
			return 1;
	}


	// initialiZe COM port if using COM5 thru COM8, the BIOS doesn't do these

	if( ucComNum > 4 ) {
		outportb( uiComBase + LCR, 0x83 ); /* Set DLAB */
		outportb( uiComBase + DLL, 12 );
		outportb( uiComBase + DLH, 0 );
		outportb( uiComBase + LCR, 0x03 ); /* Reset DLAB */
	}

	// If COM2 is connected as console using NULL modem
	// to a terminal emulator, handshaking should test ok
	// Don't test Tx & RX it will mess up the console.

	if( ucComNum == 2 )							// YES, I used a goto
		goto HandshakeOnly;						// sometimes it's the best way!

	// -------------------------------------------------------------------------
	// set RTS to tri-state RS485 receiver to avoid Rx pin contention
	// -------------------------------------------------------------------------

	outportb( (COM1BASE+MCR), 0x00 );			// Clear RTS

	// -------------------------------------------------------------------------
	// Loopback test for Transmit data connected to Receive data
	// -------------------------------------------------------------------------

	if( inportb( uiComBase + LSR ) & RCVRDY )	// Flush COM receive buffer
		inportb( uiComBase + RXR );

	if( Debug ) {
		printf("\n");
		printf("Sent    :%c%c\n", ucTestData[0], ucTestData[1] );
		printf("Received:" );
	}

	for( i=0; i<2; i++ ) {
		ucWaitCount = 0;

		// Put test data character into transmit buffer
		outportb( uiComBase + TXR, ucTestData[i] );

		// Waste time until receive buffer ready, avoid endless loops
		while( ((inportb( uiComBase+LSR ) &RCVRDY) != RCVRDY) && (ucWaitCount++ < 100) )
			delay(1);

		ucTemp1 = inportb( uiComBase + RXR );	// Read receive buffer

		if( Debug ) {
			printf("%c", ucTemp1 );				// Display receive data if debug enabled
		}

		if( ucWaitCount >= 100 ) {				// Abort if timeout reached
			printf( "\aCOM%d Test Error: Timeout waiting for receive character.\n", ucComNum );
			return 1;
		}

		if( ucTemp1 != ucTestData[i] ) {		// Abort if received data != sent data
			printf( "\aCOM%d Test Error: Character received does not match char sent.\n", ucComNum );
			return 1;
		}
	}


// -------------------------------------------------------------------------
// If COM3 is present and COM4 is not present and this is not a TS-5500
// or 
// if COM4 is present and COM5 is not present and this is a TS-5500
// then there is a TS-SER1 installed and we need to test the full handshaking
// capabilities which are:
// DTR driving DSR, DCD and RI
// or
// if there is a TS-SER4 attached and we are testing COM5-6 w/full handshaking
// -------------------------------------------------------------------------

	if( ( (ucComNum == 3) && Options.TSSER1 ) ||
		( (ucComNum == 4) && Options.TSSER1 ) ||
		( (ucComNum == 5) && Options.TSSER4 ) ||
		( (ucComNum == 6) && Options.TSSER4 ) ) {

		outportb( (uiComBase + MCR), DTR );			// Set DTR
		delay(2);									// Wait for lines to settle

		ucTemp1 = inportb( uiComBase + MSR );		// Read Modem Status Register

		if( Debug ) {
			printf("DTR set, MSR=%02X", ucTemp1 );
		}

		if( (ucTemp1 & DSR) == 0x00 ) {				// DSR should be set
			printf("\aCOM%d Test Error: handshaking DTR to DSR failed.\n", ucComNum);
			return 1;
		}
		if( (ucTemp1 & RI) == 0x00 ) {				// RI should be set
			printf("\aCOM%d Test Error: handshaking DTR to RI failed.\n", ucComNum);
			return 1;
		}
		if( (ucTemp1 & DCD) == 0x00 ) {				// DCD should be set
			printf("\aCOM%d Test Error: handshaking DTR to DCD failed.\n", ucComNum);
			return 1;
		}

		outportb( (uiComBase + MCR), 0x00 );		// Clear DTR
		delay(2);									// Wait for lines to settle

		ucTemp1 = inportb( uiComBase + MSR );		// Read Modem Status Register

		if( Debug ) {
			printf("DTR clear, MSR=%02X", ucTemp1 );
		}

		if( (ucTemp1 & DSR) == DSR ) {				// DSR should be clear
			printf("\aCOM%d Test Error: handshaking DTR to DSR failed.\n", ucComNum);
			return 1;
		}
		if( (ucTemp1 & RI) == RI ) {				// RI should be clear
			printf("\aCOM%d Test Error: handshaking DTR to RI failed.\n", ucComNum);
			return 1;
		}
		if( (ucTemp1 & DCD) == DCD ) {				// DCD should be clear
			printf("\aCOM%d Test Error: handshaking DTR to DCD failed.\n", ucComNum);
			return 1;
		}
	}

	// -------------------------------------------------------------------------
	// Handshake test for RTS connected to CTS
	// -------------------------------------------------------------------------
	HandshakeOnly:

	// TS3100 has no handshaking on COM1 
	if( (iModel == TS3100) && (ucComNum == 1) ) {
		printf( "OK\n" );
		return 0;
	}

	outportb( (uiComBase + MCR), RTS );			// Set RTS
	delay(2);									// Wait for lines to settle

	ucTemp1 = inportb( uiComBase + MSR );		// Read Modem Status Register

	if( Debug ) {
		printf("RTS set, MSR=%02X\n", ucTemp1 );
	}

	if( (ucTemp1 & CTS) == 0x00 ) {				// CTS should be set
		printf("\aCOM%d Test Error: handshaking RTS to CTS failed.\n", ucComNum);
		return 1;
	}

	outportb( (uiComBase + MCR), 0x00 );		// Clear RTS
	delay(2);									// Wait for lines to settle

	ucTemp1 = inportb( uiComBase + MSR );		// Read Modem Status Register

	if( Debug ) {
		printf("RTS clear, MSR=%02X\n", ucTemp1 );
	}

	if( (ucTemp1 & CTS) == CTS ) {				// CTS should be clear
		printf("\aCOM%d Test Error: handshaking RTS to CTS failed.\n", ucComNum);
		return 1;
	}

	printf( "OK\n" );
	return 0;
}

/****************************************************************************
   Function: test_RS485
Description: Tests RS485 hardware using external echo box.
	 Passed: COM port to test
	Returns: int - result of test, 0=success, 1=failure
Revised mtj 20021022 passes COM port, was hard coded to COM1
*****************************************************************************/
int test_RS485( unsigned char ucComNum )
{

	unsigned int uiComBase;
	unsigned char ucCTSasserted = FALSE;
	unsigned char ucWaitCount = 0;
	unsigned char ucTestData[2] = { 0x7A, 0x25};
	unsigned char ucFullDuplexPassed = FALSE;

	printf( "Testing RS485 on COM%d : ", ucComNum );

	// -------------------------------------------------------------------------
	// Setup com port base address
	// -------------------------------------------------------------------------

	switch( ucComNum ) {
		case 1: uiComBase = COM1BASE;   break;
		case 2: uiComBase = COM2BASE;   break;
		case 3: uiComBase = COM3BASE;   break;
		case 4: uiComBase = COM4BASE;   break;
		case 5: uiComBase = COM5BASE;   break;
		case 6: uiComBase = COM6BASE;   break;
		case 7: uiComBase = COM7BASE;   break;
		case 8: uiComBase = COM8BASE;   break;
		default:
			printf("\aCom%d Test Error: Invalid port number\n", ucComNum );
			return 1;
	}

	// -------------------------------------------------------------------------
	// Check RS485 option installed
	// -------------------------------------------------------------------------

	if( !(Options.RS485) && !(Options.TSSER4_RS485) ) {		// if RS485 option disabled in PLD
		printf("\aError: RS485 option not installed\n");
		return 1;
	}

	// initialiZe COM port if using COM5 thru COM8, the BIOS doesn't do these

	if( ucComNum > 4 ) {
		outportb( uiComBase + LCR, 0x83 ); /* Set DLAB */
		outportb( uiComBase + DLL, 12 );
		outportb( uiComBase + DLH, 0 );
		outportb( uiComBase + LCR, 0x03 ); /* Reset DLAB */
	}

	// TS3100 has no handshaking on COM1, can't detect loopback, half duplex only
	if( iModel == TS3100 )
		goto Half_Duplex;

	// set bit to enable RTS on TS-5x00 products. (benign to other products)
	outportb( PLD_OPT1_ADDR, 0x40 );

	// -------------------------------------------------------------------------
	// checking for loopback cable attached, CTS follows RTS if installed.
	// -------------------------------------------------------------------------

	outportb( (uiComBase+MCR), RTS );			// Set RTS
	delay(2);									// Wait for lines to settle

	ucTemp1 = inportb( uiComBase + MSR );		// Read Modem Status Register

	if( (ucTemp1 & CTS) == CTS )				// Check if CTS active
		ucCTSasserted = TRUE;					// Set CTS asserted flag

	outportb( (uiComBase+MCR), 0x00 );			// Clear RTS
	delay(2);									// Wait for lines to settle

	ucTemp1 = inportb( uiComBase+MSR );			// read Modem Status Register

	// If CTS follows RTS, abort test
	if( ((ucTemp1 & CTS) == 0x00) && (ucCTSasserted) ) {
		printf("\aError: COM%d Loopback cable detected, remove.\n", ucComNum);
		return 1;
	}


	outportb( (uiComBase+MCR), RTS );			// Set RTS to enable RS485 transmit
	delay(2);									// Wait for lines to settle

	for( i=0; i<10; i++ )						// Flush receive buffer very well
		if( inportb( uiComBase+LSR ) & RCVRDY )
			inportb( uiComBase+RXR );


	if( Debug ) {
		printf("Sent    :%c%c\n", ucTestData[0], ucTestData[1] );
		printf("Received:" );
	}

	for( i=0; i<2; i++ ) {
		ucWaitCount = 0;

		// Put test data character into transmit buffer
		outportb( uiComBase + TXR, ucTestData[i] );

		// Waste time until receive buffer ready, avoid endless loops
		while( ((inportb( uiComBase+LSR ) &RCVRDY) != RCVRDY) && (ucWaitCount++ < 100) )
			delay(1);

		ucTemp1 = inportb( uiComBase + RXR );	// Read receive buffer

		if( Debug ) {
			printf("%c", ucTemp1 );				// Display receive data if debug enabled
		}

		if( ucWaitCount >= 100 ) {				// Error if timeout reached
//			printf( "\aError: No character received.\n" );
			ucFullDuplexPassed = FALSE;
			//   return 1;
		} else if( ucTemp1 != ucTestData[i] ) {	// Error if received data != sent data
//			printf( "\aError: Character received does not match char sent.\n" );
			ucFullDuplexPassed = FALSE;
			//   return 1;
		} else {								// No errors, must be full duplex
			ucFullDuplexPassed = TRUE;
		}
	}

	if( ucFullDuplexPassed == TRUE ) {
		printf( "OK - Full duplex\n" );
		return 0;
	}


	//-------------------------------------------------------------------------
	// Loop to send out 12 test pattern characters to RS485 echo box, this box
	// stores the 12 characters, waits 100 mSec, then transmits them back to us.
	// -------------------------------------------------------------------------
	Half_Duplex:

	outportb( (uiComBase+MCR), RTS );			// Set RTS to enable RS485 transmit
	delay(2);									// Wait for lines to settle

	for( i=0; i<10; i++ )						// Flush receive buffer very well
		if( inportb( uiComBase+LSR ) & RCVRDY )
			inportb( uiComBase+RXR );

	ucWaitCount = 0;

	if( Debug ) {
		printf("Sent   :\n" );
	}

	for( i=0; i<12; i++ ) {
		ucTemp1 = 30+7*i;						// Generate test character to send

		if( Debug ) {
			printf("%c", ucTemp1 );				// Display pattern if debug enabled
		}

		outportb( uiComBase+TXR,ucTemp1);		// Put character in transmit buffer.
		delay(3);								// Wait for char to transmit.
	}

	outportb( (uiComBase+MCR), 0x00 );			// Clear RTS to allow RS485 to receive
	delay(2);									// Wait for lines to settle

	// -------------------------------------------------------------------------
	// Loop to receive 12 test pattern characters back from RS485 echo box.
	// -------------------------------------------------------------------------

	if( Debug ) {
		printf("\nReceived:" );
	}

	for( i=0; i<12; i++ ) {
		// Waste time until Receive buffer ready, avoid endless loops
		while( ((inportb( uiComBase+LSR ) &RCVRDY) != RCVRDY) && (ucWaitCount++ < 100) )
			delay(1);

		ucTemp1 = inportb( uiComBase+RXR );		// Read receive buffer

		if( Debug ) {
			printf("%c", ucTemp1 );				// Display receive data if debug enabled
		}

		if( ucWaitCount >= 100 ) {				// Abort if timeout reached
			printf( "\aError: Timeout waiting for response echo box.\n" );
			return 1;
		}

		if( ucTemp1 != 30+7*i ) {				// compare receive data to sent data
			printf( "\aError: Invalid response from echo box.\n" );
			return 1;
		}
	}

	printf( "OK - Half duplex\n" );
	return 0;

}
/****************************************************************************
	Function: test_socket
 Description: determine socket location and test SRAM in JEDEC socket
	  Passed: type of test, normal or battery backed test
	 Returns: int - result of test, 0=success, 1=failure
*****************************************************************************/
int test_socket( unsigned char TestType )
{

	int uiSRAMsegment;
	int uiSRAMlength;

	unsigned char far *loc1;
	unsigned char far *loc2;

	int z=0;
	int errors=0;

	unsigned char low_page_done = FALSE;

	if( TestType == BATTERY_SRAM )
		printf( "Testing Socket - Battery backup - " );
	else
		printf( "Testing Socket : " );

	if( Options.SRAM ) {  // if SRAM option enabled in PLD
		uiSRAMlength = 0x7F00;		 // socket window is 32k

		// find 32K window location in memory map for this particular product

		switch( iModel ) {
			case TS2350:    uiSRAMsegment = 0xC800; break;
			case TS2100:
			case TS2200:    uiSRAMsegment = 0xC800; break;
			case TS2800:    uiSRAMsegment = 0xD000; break;
			case TS3100:
			case TS3200:
			case TS3300:
			case TS3400:    uiSRAMsegment = 0xD800; break;
			case TS5300:
			case TS5400:
			case TS5500:
			case TS5600:
			case TS5700:    uiSRAMsegment = 0xD000; break;
		}

		if( Options.SRAM_128K ) {	// if 128 KB SRAM option enabled in PLD

			outportb( 0x77, 0x00 );	// clear page bit

//			uiSRAMlength = 0xFF00;	// socket window is 64 KB x 2 pgs
			uiSRAMlength = 0x7F00;	// test half to avoid total rewrite

			// find 64K window (x2 pages) location in memory map for this particular product

			switch( iModel ) {
				case TS3100:
				case TS3200:
				case TS3300:
				case TS3400:    uiSRAMsegment = 0xC400; break;
			}
		}

	} else {								  // if no SRAM option
		uiSRAMlength = 0x1F00;		 // socket window is 8k for DiskOnChip millenium

		// find 8K window location in memory map for this particular product

		switch( iModel ) {
			case TS2350:    uiSRAMsegment = 0xCE00; break;
			case TS2100:
			case TS2200:    uiSRAMsegment = 0xCE00; break;
			case TS2800:    uiSRAMsegment = 0xD600; break;
			case TS3100:
			case TS3200:
			case TS3300:
			case TS3400:    uiSRAMsegment = 0xD800; break;
			case TS5300:
			case TS5400:
			case TS5500:
			case TS5600:
			case TS5700:    uiSRAMsegment = 0xD000; break;
		}
	}

	// setup pointers to first two locations in window to check for DiskOnChip
	// installed in the socket

	loc1 = (unsigned char far *) MK_FP ( uiSRAMsegment, 0x0000 );
	loc2 = (unsigned char far *) MK_FP ( uiSRAMsegment, 0x0001 );

	// If a DOC is installed first two locations will have the x55 and xAA codes
	// required for BIOS extension ROMs, abort socket test.

	if( (*loc1 == 0x55) && (*loc2 == 0xAA) ) {
		printf("\aDisk on chip found...aborting test\n");
		return 1;
	}

	SRAM_test_start:

	z = 0;

	if( Debug ) {
		printf("\nSRAM seg = %04X SRAM len = %04X\\n", uiSRAMsegment, uiSRAMlength );
	}

	// write pattern to SRAM, unless are testing the battery backed SRAM.

	if( TestType != BATTERY_SRAM ) {
		for( i=0x0000; i<uiSRAMlength; i=i+15 ) {
			z = (z +37) % 256;

			loc1 =(unsigned char far *) MK_FP( uiSRAMsegment, i );

			*loc1 = z;
		}
	}

	z=0;

	// read pattern from SRAM and verify reads equal what was written

	for( i=0x0000; i<uiSRAMlength; i=i+15 ) {
		z = (z +37) % 256;

		loc1 = (unsigned char far *) MK_FP ( uiSRAMsegment, i );

		if( *loc1 != (z) ) {
			errors++;
			if( Debug ) {
				printf("%p=%d\n", (unsigned char far *)loc1, z );
			}
		}
	}


	// if 128 KB SRAM installed and page bit still at zero,
	// set page bit and test next 64 KB window
	if( (Options.SRAM_128K) && (low_page_done==0) ) {
		low_page_done = TRUE;

		// store page check byte on page 0
		loc1 = (unsigned char far *) MK_FP ( uiSRAMsegment, 0x0005 );
		*loc1 = '0';

		outportb( 0x77, 0x01 );		// set page bit to page 1

		// store page check byte on page 1
		*loc1 = '1';

		if( Debug )
			printf( "\n128 KB SRAM - testing page two\n" );

		goto SRAM_test_start;		// repeat test
	}

	if( Options.SRAM_128K ) {
		// look for page check byte on page 1, shouldn't be available on 2nd page
		loc1 = (unsigned char far *) MK_FP ( uiSRAMsegment, 0x0005 );
		if( (*loc1 == '0') || (*loc1 != '1') ) {
			printf("\aSRAM test error: page switch failure\n" );
			return 1;
		}
	}


	// return 1 if errors, 0 if no errors

	if( errors == 0 ) {
		printf( "OK\n" );
		return 0;
	} else {
		printf("\aSRAM test error: data write/read not equal.\n" );
		return 1;
	}

}
/****************************************************************************
	Function: out_lcd_wait_for_busy
 Description: Send data to LCD and wait for transfer to complete
	  Passed: LCD port to write, data to write
	 Returns: none
*****************************************************************************/

void out_lcd_wait_for_busy( unsigned char port, unsigned char data )
{
	unsigned char no_hang = 5;
	unsigned char result;

	outportb( port, data );

	do {
		delay(1);
		result = inportb( LCD_CTRL );
		result &= 0x80;
	} while( (result) && (--no_hang) );
}

/****************************************************************************
	Function: test_LCD
 Description: Use special BIOS call and direct I/O access to test LCD port
	  Passed: none
	 Returns: int - result of test, 0=success, 1=failure 
*****************************************************************************/
int test_LCD( void )
{
	unsigned char result;
	unsigned char ucCount = 3;

	ucError = FALSE;

	printf( "Testing LCD I/O mode : " );

	// check for 9500 with DIP #8 on to enable video, if so abort LCD test
	if( inportb( 0x19D ) == 95 ) {

		if( ( inportb( 0x19B ) & 0x80 ) ) {
			printf( "\aTS-9500 video detected, aborting LCD test\n" );
			return 1;
		}
	}

	outportb( 0x7d, 0x10 );		// put into LCD mode

	out_lcd_wait_for_busy( LCD_CTRL, 0x30 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x30 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x30 );

	result = inportb( LCD_CTRL );

	if( result > 127 ) {
		printf( "\aLCD not detected\n" );
		return 1;
	}


	out_lcd_wait_for_busy( LCD_CTRL, 0x38 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x08 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x01 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x06 );
	out_lcd_wait_for_busy( LCD_CTRL, 0x0E );	// Liberty's way
//	out_lcd_wait_for_busy( LCD_CTRL, 0x0C );	// Bob's way
//	out_lcd_wait_for_busy( LCD_CTRL, 0x06 );	// Bob's way
	out_lcd_wait_for_busy( LCD_CTRL, 0x01 );

	out_lcd_wait_for_busy( LCD_DATA, 'T' );
	out_lcd_wait_for_busy( LCD_DATA, '-' );
	out_lcd_wait_for_busy( LCD_DATA, 'S' );
	out_lcd_wait_for_busy( LCD_DATA, 'Y' );
	out_lcd_wait_for_busy( LCD_DATA, 'S' );
	out_lcd_wait_for_busy( LCD_DATA, 'T' );
	out_lcd_wait_for_busy( LCD_DATA, 'E' );
	out_lcd_wait_for_busy( LCD_DATA, 'M' );
	out_lcd_wait_for_busy( LCD_DATA, 'S' );

	printf( "OK\n" );

	printf ("Attempting LCD console mode : ");

	// call BIOS function x15, subfunction xB042 to redirect console to LCD

	regs.x.ax = 0xB042;
	regs.h.bh = 0x01;
	int86 (0x15, &regs, &regs);

	// if result is good, redirect console to LCD, else show error

	if( regs.h.ah == 0 && regs.h.bh == 1 ) {
		printf ("console mode OK : testing stdout : ");
		SetConsolePort (0);			 // turn off COM2 redirection for LCD display
		redirect = 1;				 // set flag to undo redirection later
	} else {
		ucError = TRUE;
		redirect = 0;
		printf ("\aFailed - ");
		switch( regs.h.ah ) {
			case 1: printf ("no LCD interface present.\n"); break;
			case 3: printf ("LCD plugged in?\n");           break;
			default:    printf ("** Unknown error **\n");   break;
		}
		return 1;
	}

/*
do{
	regs.x.ax = 0xB042;
	regs.h.bh = 0x02;
	int86 (0x15, &regs, &regs);

	if( regs.h.bh == 1 )
		outportb( 0x77, 0x01 );
	else
		outportb( 0x77, 0x00 );


	printf( "lcd status = %02X Enable = %02X\n", regs.h.ah, regs.h.bh );
	}
	while( (regs.h.bh == 0x01) && (! kbhit()) );
*/

	while( ucCount-- && redirect )
	//		while (!GetPswStatus() && !kbhit())
	//		while (!kbhit() && redirect)
	//		while(1)
	{
		gettime (&t);
		getdate (&d);

		system ("cls");

		printf ("LCD test for 3 seconds\n");
		printf ("  %2d:%02d:%02d  %d-%02d-%02d ",
				t.ti_hour, t.ti_min, t.ti_sec,
				d.da_year, d.da_mon, d.da_day);

		sleep (1);
	}


	if( Options.Keypad )
		test_keypad();


	// If an LCD interface is enabled, disable it now
	if( redirect ) {
		printf ("Attempting LCD disable... ");
//		system ("cls");
		regs.x.ax = 0xB042;	// turn off LCD
		regs.h.bh = 0x00;
		int86 (0x15, &regs, &regs);

		if( regs.h.ah == 0 && regs.h.bh == 0 ) {
			printf ("Success!\n");
			SetConsolePort (2);
		} else {
			printf ("Failed.\n");
			ucError = TRUE;
		}

	}

	if( ucError ) {
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}
}

/****************************************************************************
	Function: test_keypad
 Description: test matrix keypad using BIOS call
	  Passed: none
	 Returns: zero always
*****************************************************************************/
int test_keypad( void )
{
	unsigned char ucCount = 0;
	char ucKeypress = 0;

	printf ("\nHit 16 keys or D\n");

	// call BIOS function x15, subfunction xB040 to redirect console to keypad

	regs.x.ax = 0xB040;					// TS BIOS matrix keypad function
	regs.h.bh = 0x01;					// keypad on
	int86 (0x15, &regs, &regs);			// Call to INT 15 function xB000

	SetConsolePort (0);

// this will get kepad keypress and put them to the console

	while( (ucCount++ < 16) && (ucKeypress != 'D') ) {
		ucKeypress = getch();
		printf( "%c", ucKeypress );
	};

	printf ("Keypad disable... ");

	// call BIOS function x15, subfunction xB040 to redirect console to COM2

	regs.x.ax = 0xB040;					// TS BIOS matrix keypad function
	regs.h.bh = 0x00;					// keypad off
	int86 (0x15, &regs, &regs);			// Call to INT 15 function xB000

	// this may be wrong if TS-9500 is installed!

	printf ("Console to COM2\n");
	SetConsolePort (2);

	return 0;
}

/****************************************************************************
   Function: test_DIO_5600
Description: DIO test routine which works with TS-5600 and TS-5700
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO_5600( void )
{

	unsigned char UpperByte;
	unsigned char LowerByte;

	printf( "Testing DIO2 : " );

// Test DIO Port     (Loop Back Test)
//
// IS-------
//  OUT    IN
//
//   8  -- 0,7
//   9  -- 1,6,13 (13 is now GPIRQ6)
//   10 -- 2,5
//   11 -- 3,4
//	
//	12 is external reset
//
// WAS-------
//   8  -- 0,7
//   9  -- 1,6,13
//   10 -- 2,5
//   11 -- 3,4
//	
//	12 is external reset

// Loop Back Cable must connect the above lines

	ucError = FALSE;

	// set all I/O pins to inputs

	outportb( PLD_DIO2_CTRL, 0x00 );

	// set outputs 8-11 high, all inputs should be high

	outportb( PLD_DIO2_HIGH, 0xFF );
	delay(1);

	// all I/O should be high

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0xFF ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set outputs 8-11 low, all inputs should low


	outportb( PLD_DIO2_HIGH, 0x0);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0x00 ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on all I/O set to 0\n");
		}
	}

	// set only output 8 low, only inputs 0 and 7 should low

	outportb( PLD_DIO2_HIGH, 0x0E);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0x7E ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on I/O 8 set to 0\n");
		}
	}

	// set only output 9 low, only inputs 1,6 should be low

	outportb( PLD_DIO2_HIGH, 0x0D);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0xBD ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on I/O 9 set to 0\n");
		}
	}

	// set only output 10 low, only inputs 2,5 should be low

	outportb( PLD_DIO2_HIGH, 0x0B);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0xDB ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on I/O 10 set to 0\n");
		}
	}
	// set only output 11 low, only inputs 3,4 should low

	outportb( PLD_DIO2_HIGH, 0x07);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );

	if( LowerByte != 0xE7 ) {
		ucError = TRUE;
		if( Debug ) {
			printf(" Lower=%02X\n",  LowerByte );
			printf("I/O test failed on I/O 11 set to 0\n");
		}
	}

	// report error

	if( ucError ) {
		printf("\aDIO2 I/O failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}

}


/****************************************************************************
   Function: test_DIO2_5500
Description: DIO test routine which avoids triggering external reset 54/5500
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO2_5500( void )
{

	unsigned char UpperByte;
	unsigned char LowerByte;

	printf( "Testing DIO2 : " );

// Test DIO2 Port     (Loop Back Test)
//
//  OUT    IN
//
//   8  -- 0,7
//   9  -- 1,6,13
//   10 -- 2,5
//   11 -- 3,4
//	
//	12 is external reset
//
// Loop Back Cable must connect the above lines

	ucError = FALSE;

	// set all I/O pins to inputs, to check pullups.

	outportb( PLD_DIO2_CTRL, 0x00 );
	delay(1);

	// all I/O should be high, all have pullups

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x2F;

	if( (LowerByte != 0xFF) || (UpperByte != 0x2F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set I/O 0-7 as inputs, 8-11 as outputs, 12-13 are always inputs

	outportb( PLD_DIO2_CTRL, 0x20 );
	delay(1);

	// set outputs 8-11 low, all inputs should low


	outportb( PLD_DIO2_HIGH, 0x0);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x20;

	if( (LowerByte != 0x00) || (UpperByte != 0x00) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on all I/O set to 0\n");
		}
	}

	// set only output 8 low, only inputs 0 and 7 should low

	outportb( PLD_DIO2_HIGH, 0x0E);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x20;

	if( (LowerByte != 0x7E) || (UpperByte != 0x20) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 8 set to 0\n");
		}
	}

	// set only output 9 low, only inputs 1,6,13 should low

	outportb( PLD_DIO2_HIGH, 0x0D);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x20;

	if( (LowerByte != 0xBD) || (UpperByte != 0x00) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 9 set to 0\n");
		}
	}

	// set only output 10 low, only inputs 2,5,12 should low

	outportb( PLD_DIO2_HIGH, 0x0B);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x20;

	if( (LowerByte != 0xDB) && (UpperByte != 0x20) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 10 set to 0\n");
		}
	}
	// set only output 11 low, only inputs 3,4 should low

	outportb( PLD_DIO2_HIGH, 0x07);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x20;

	if( (LowerByte != 0xE7) || (UpperByte != 0x20) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 11 set to 0\n");
		}
	}

	// report error

	if( ucError ) {
		printf("\aDIO2 I/O failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}

}


/****************************************************************************
   Function: test_DIO1_5300
Description: DIO1 test routine for TS-5300 product
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO1_5300( void )
{

	unsigned char UpperByte;
	unsigned char LowerByte;

	printf( "Testing DIO1 : " );

// Test DIO1 Port     (Loop Back Test)
//
//  OUT    IN
//
//   8  -- 0,7
//   9  -- 1,6,13
//   10 -- 2,5,12
//   11 -- 3,4
//
// Loop Back Cable must connect the above lines

	ucError = FALSE;

	// set all I/O pins to inputs, to check pullups.

	outportb( PLD_DIO1_CTRL, 0x00 );
	delay(1);

	// all I/O should be high, all have pullups

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x3F;

	if( (LowerByte != 0xFF) || (UpperByte != 0x3F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set I/O 0-7 as inputs, 8-11 as outputs, 12-13 are always inputs

	outportb( PLD_DIO1_CTRL, 0x20 );
	delay(1);

	// set outputs 8-11 low, all inputs should low

	outportb( PLD_DIO1_HIGH, 0x00);
	delay(1);

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x30;

	if( (LowerByte != 0x00) || (UpperByte != 0x00) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on all I/O set to 0\n");
		}
	}

	// set only output 8 low, only inputs 0 and 7 should low

	outportb( PLD_DIO1_HIGH, 0x0E);
	delay(1);

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x30;

	if( (LowerByte != 0x7E) || (UpperByte != 0x30) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 8 set to 0\n");
		}
	}

	// set only output 9 low, only inputs 1,6,13 should low

	outportb( PLD_DIO1_HIGH, 0x0D);
	delay(1);

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x30;

	if( (LowerByte != 0xBD) || (UpperByte != 0x10) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 9 set to 0\n");
		}
	}
	// set only output 10 low, only inputs 2,5,12 should low

	outportb( PLD_DIO1_HIGH, 0x0B);
	delay(1);

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x30;

	if( (LowerByte != 0xDB) && (UpperByte != 0x20) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 10 set to 0\n");
		}
	}
	// set only output 11 low, only inputs 3,4 should low

	outportb( PLD_DIO1_HIGH, 0x07);
	delay(1);

	LowerByte = inportb( PLD_DIO1_LOW );
	UpperByte = inportb( PLD_DIO1_HIGH ) & 0x30;

	if( (LowerByte != 0xE7) || (UpperByte != 0x30) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 11 set to 0\n");
		}
	}

	// report error

	if( ucError ) {
		printf("\aDIO1 I/O failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}


}

/****************************************************************************
   Function: test_DIO2_5500
Description: DIO2 test routine for TS-5300
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO2_5300( void )
{

	unsigned char UpperByte;
	unsigned char LowerByte;

	printf( "Testing DIO2 : " );

// Test DIO2 Port     (Loop Back Test)
//
//  OUT    IN
//
//   8  -- 0,7
//   9  -- 1,6,13
//   10 -- 2,5,12
//   11 -- 3,4
//
// Loop Back Cable must connect the above lines

	ucError = FALSE;

	// set all I/O pins to inputs, to check pullups.

	outportb( PLD_DIO2_CTRL, 0x00 );
	delay(1);

	// all I/O should be high, all have pullups

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x3F;

	if( (LowerByte != 0xFF) || (UpperByte != 0x3F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set I/O 0-7 as inputs, 8-11 as outputs, 12-13 are always inputs

	outportb( PLD_DIO2_CTRL, 0x20 );
	delay(1);

	// set outputs 8-11 low, all inputs should low


	// mtj 20021111 Bob has external reset capability hard wired into DIO2.13 on the 
	// TS-5400 and TS-5500.

	if( !(Options.EXTRESET) ) {

		outportb( PLD_DIO2_HIGH, 0x0);
		delay(1);

		LowerByte = inportb( PLD_DIO2_LOW );
		UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

		if( (LowerByte != 0x00) || (UpperByte != 0x00) ) {
			ucError = TRUE;
			if( Debug ) {
				printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
				printf("I/O test failed on all I/O set to 0\n");
			}
		}
	} else {

		outportb( PLD_DIO2_HIGH, 0x00);
		delay(1);

		LowerByte = inportb( PLD_DIO2_LOW );
		UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

		if( (LowerByte != 0x00) || (UpperByte != 0x00) ) {
			ucError = TRUE;
			if( Debug ) {
				printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
				printf("I/O test failed on all I/O set to 0\n");
			}
		}
	}

	// set only output 8 low, only inputs 0 and 7 should low

	outportb( PLD_DIO2_HIGH, 0x0E);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

	if( (LowerByte != 0x7E) || (UpperByte != 0x30) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 8 set to 0\n");
		}
	}

	// set only output 9 low, only inputs 1,6,13 should low

	if( !(Options.EXTRESET) ) {

		outportb( PLD_DIO2_HIGH, 0x0D);
		delay(1);

		LowerByte = inportb( PLD_DIO2_LOW );
		UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

		if( (LowerByte != 0xBD) || (UpperByte != 0x10) ) {
			ucError = TRUE;
			if( Debug ) {
				printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
				printf("I/O test failed on I/O 9 set to 0\n");
			}
		}
	}

	// set only output 10 low, only inputs 2,5,12 should low

	outportb( PLD_DIO2_HIGH, 0x0B);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

	if( (LowerByte != 0xDB) && (UpperByte != 0x20) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 10 set to 0\n");
		}
	}
	// set only output 11 low, only inputs 3,4 should low

	outportb( PLD_DIO2_HIGH, 0x07);
	delay(1);

	LowerByte = inportb( PLD_DIO2_LOW );
	UpperByte = inportb( PLD_DIO2_HIGH ) & 0x30;

	if( (LowerByte != 0xE7) || (UpperByte != 0x30) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("Upper=%02X, Lower=%02X\n", UpperByte, LowerByte );
			printf("I/O test failed on I/O 11 set to 0\n");
		}
	}

	// report error

	if( ucError ) {
		printf("\aDIO2 I/O failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}

}

/****************************************************************************
   Function: test_DIO2_3200
Description: DIO2 test routine for TS-3x00 family products
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO2_3200( void )
{

	unsigned char ucP1CFGsave = inportb( P1CFG );	// restore after tests
	unsigned char ucP3CFGsave = inportb( P3CFG );	// restore after tests

	printf( "Testing DIO2 : " );

// Test DIO2 Port     (Loop Back Test)
//
//    OUT       IN
//
//   DIO2_7 -- DIO2_0
//   DIO2_6 -- P1.3
//   DIO2_5 -- P3.4
//   DIO2_4 -- DCD2
//   P1.1   -- DIO2_3
//   P1.2   -- DIO2_2
//   P1.6   -- DIO2_1
//
// Loop Back Cable must connect the above lines


	// initialize port 3 I/O pins ==============================================

	//	set port 3 pin 4 to I/O mode (0=I/O mode, 1=peripheral mode)
	// set port 3 pin 4 as input (0=comp. output, 1=input/open-drain output) 
	// set port 3 pin 4 to a logic one

	outportb( P3CFG, (inportb(P3CFG) & 0xEF) );

	outportb( P3DIR, 0xFF );

	outportb( P3LTC, 0xFF );

	// initialize port 1 I/O pins ==============================================

	//	set port 1 pins 1,2,3,6 to I/O mode (0=I/O mode, 1=peripheral mode)
	// set port 1 pins 1,2,6 as outputs, pin 3 as input (6 is inverted LED)
	// set port 1 pins 1,2,3,6 to a logic one

	outportb( P1CFG, (inportb(P1CFG) & 0xD8) );

	outportb( P1DIR, 0xB9 );

	outportb( P1LTC, 0xBF );

	// initialize DIO2 I/O pins ==============================================

	// set DIO2 pins 0-3 as inputs, pins 4-7 as outputs
	// set DIO2 pins 0-7 to a logic one

	outportb( PLD_DIO2_CTRL, 0x02 );

	outportb( PLD_DIO2_LOW, 0xFF );

	// test I/O pins ===========================================================

	ucError = FALSE;

	// all outputs set to one, test all inputs for one
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set only port 1 bit 1 to zero, only DIO2 bit 3 should be zero
	//depends on previous test passing
	outportb( P1LTC, 0xBD );
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x07) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on P1.1 to DIO2.3 set to 0\n");
		}
	}

	// set only port 1 bit 2 to zero, only DIO2 bit 2 should be zero

	outportb( P1LTC, 0xBB );
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0B) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on P1.2 to DIO2.2 set to 0\n");
		}
	}

	// set only port 1 bit 6 to zero, only DIO2 bit 1 should be zero
	// port 1 bit 6 is inverted and controls the LED.

	outportb( P1LTC, 0xFF ); 
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0D) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on P1.6 to DIO2.1 set to 0\n");
		}
	}

	// set only DIO2 bit 4 to zero, only DCD2 should be zero
	//3400 with External Reset...If the Options says we have external reset, then doing this test will
	//cause the board to reboot itself.
	if( !(Options.EXTRESET) ) {
		outportb( P1LTC, 0xBF );
		outportb( PLD_DIO2_LOW, 0xEF );
		delay(100);

		ucTemp1 = inportb(P1PIN) & 0x08;
		ucTemp2 = inportb(P3PIN) & 0x10;
		ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
		ucTemp4 = inportb(COM2BASE+MSR) & DCD;
		if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0F) || (ucTemp4 != DCD) ) {
			ucError = TRUE;
			if( Debug ) {
				printf("P1=%02X, P3=%02X, D2=%02X MSR=%02X\n", ucTemp1, ucTemp2, ucTemp3, ucTemp4 );
				printf("I/O test failed on DIO2.4 to DCD2 set to 0\n");
			}
		}
	}

	// set only DIO2 bit 5 to zero, only port 3 bit 4 should be zero

	outportb( PLD_DIO2_LOW, 0xDF );
	outportb( P1LTC,0xBF);
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x00) || (ucTemp3 != 0x0F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on DIO2.5 to P3.4 set to 0\n");
		}
	}

	// set only DIO2 bit 6 to zero, only port 1 bit 3 should be zero

	outportb( PLD_DIO2_LOW, 0xBF );
	outportb (P1LTC,0XBF);
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x00) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0F) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on DIO2.6 to P1.3 set to 0\n");
		}
	}

	// set only DIO2 bit 7 to zero, only DIO2 bit 0 should be zero

	outportb( PLD_DIO2_LOW, 0x7F );
	outportb (P1LTC,0XBF);
	delay(100);

	ucTemp1 = inportb(P1PIN) & 0x08;
	ucTemp2 = inportb(P3PIN) & 0x10;
	ucTemp3 = inportb(PLD_DIO2_LOW) & 0x0F;
	if( (ucTemp1 != 0x08) || (ucTemp2 != 0x10) || (ucTemp3 != 0x0E) ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P1=%02X, P3=%02X, D2=%02X\n", ucTemp1, ucTemp2, ucTemp3 );
			printf("I/O test failed on DIO2.7 to DIO2.0 set to 0\n");
		}
	}


	outportb( P1CFG, ucP1CFGsave );						// restore as found
	outportb( P3CFG, ucP3CFGsave );						// restore as found

	if( ucError ) {
		printf("\aDIO2 I/O failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;
	}

}

/****************************************************************************
   Function: test_DIO1_3200
Description: DIO1 test routine for Ts-3x00 family products
	 Passed: none
	Returns: 1 on error, 0 on success
*****************************************************************************/
int test_DIO1_3200( void )
{

	unsigned char ucP1CFGsave = inportb( P1CFG );	// restore after tests
	unsigned char ucP3CFGsave = inportb( P3CFG );	// restore after tests

	printf( "Testing DIO1 : " );

// Test DIO1 Port     (Loop Back Test)
//
//    OUT     IN
//
//   P3.2 -- P3.3
//   P3.6 -- P3.1
//   P1.0 -- P3.5
//   P1.5 -- P3.0
//   DTR2 -- RI2
//   RTS2 -- DSR2
//
// Loop Back Cable must connect the above lines


	// initialize port 3 I/O pins ==============================================

	//	set port 3 pins 0,1,2,3,5,6 to I/O mode (0=I/O mode, 1=peripheral mode)
	// set port 3 pins 2,6 as outputs, pins 0,1,3,5 as inputs
	// set port 3 pins 0,1,2,3,5,6 to a logic one

	// bit 3 = IRQ5 - keyboard on MPC204-9, change to 0x98
	outportb( P3CFG, (inportb(P3CFG) & 0x90) );

	outportb( P3DIR, 0xBB );

	outportb( P3LTC, 0xFF );

	// initialize port 1 I/O pins ==============================================

	//	set port 1 pins 0,5 to I/O mode (0=I/O mode, 1=peripheral mode)
	// set port 1 pins 0,5 as outputs (0=comp. output, 1=input/open-drain output)
	// set port 1 pins 0,5 to a logic one

	outportb( P1CFG, (inportb(P1CFG) & 0xDE) );

	outportb( P1DIR, 0xDE );

	outportb( P1LTC, 0xFF );

	// test I/O pins ===========================================================

	ucError = FALSE;

	// all outputs set to one, test all inputs for one
	delay(100);

	ucTemp1 = inportb(P3PIN) & 0x2B;
	ucTemp2 = inportb(P1PIN) & 0x21;
	if( ucTemp1 != 0x2B ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P3=%02X, P1=%02X\n", ucTemp1, ucTemp2 );
			printf("I/O test failed on all I/O set to 1\n");
		}
	}

	// set only port 3 bit 2 to zero, only port 3 bit 3 should be zero

	outportb( P3LTC, 0xFB );
	delay(100);

	ucTemp1 = inportb(P3PIN) & 0x2B;
	ucTemp2 = inportb(P1PIN) & 0x21;
	if( ucTemp1 != 0x23 ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P3=%02X, P1=%02X\n", ucTemp1, ucTemp2 );
			printf("I/O test failed on P3.2 to P3.3 set to 0\n");
		}
	}

	// set only port 3 bit 6 to zero, only port 3 bit 1 should be zero

	outportb( P3LTC, 0xBF );
	delay(100);

	ucTemp1 = inportb(P3PIN) & 0x2B;
	ucTemp2 = inportb(P1PIN) & 0x21;
	if( ucTemp1 != 0x29 ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P3=%02X, P1=%02X\n", ucTemp1, ucTemp2 );
			printf("I/O test failed on P3.6 to P3.1 set to 0\n");
		}
	}

	// set only port 1 bit 0 to zero, only port 3 bit 5 should be zero

	outportb( P3LTC, 0xFF );
	outportb( P1LTC, 0xFE );
	delay(100);

	ucTemp1 = inportb(P3PIN) & 0x2B;
	ucTemp2 = inportb(P1PIN) & 0x21;
	if( ucTemp1 != 0x0B ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P3=%02X, P1=%02X\n", ucTemp1, ucTemp2 );
			printf("I/O test failed on P1.0 to P3.5 set to 0\n");
		}
	}

	// set only port 1 bit 5 to zero, only port 3 bit 0 should be zero

	outportb( P1LTC, 0xDF );
	delay(100);

	ucTemp1 = inportb(P3PIN) & 0x2B;
	ucTemp2 = inportb(P1PIN) & 0x21;
	if( ucTemp1 != 0x2A ) {
		ucError = TRUE;
		if( Debug ) {
			printf("P3=%02X, P1=%02X\n", ucTemp1, ucTemp2 );
			printf("I/O test failed on P1.5 to P3.0 set to 0\n");
		}
	}

	// report error

	if( ucError ) {
		printf("\aDIO1 I/O failure\n");
		return 1;
	}

	// test handshake pins =====================================================

	outportb( (COM2BASE+MCR), RTS );						// set RTS
	delay(100);

	ucTemp1 = inportb(COM2BASE+MSR);
	if( (ucTemp1 & DSR) != DSR ) {							// DSR should be one
		ucError = TRUE;
		if( Debug ) {
			printf("MSR=%02X\n", ucTemp1 );
			printf("Handshake test failed on RTS to DSR set to 1\n");
		}
	}

	outportb( (COM2BASE+MCR), 0x00 );			  // clear RTS
	delay(100);

	ucTemp1 = inportb(COM2BASE+MSR);
	if( ucTemp1 & DSR ) {									   // DSR should be zero
		ucError = TRUE;
		if( Debug ) {
			printf("MSR=%02X\n", ucTemp1 );
			printf("Handshake test failed on RTS to DSR set to 0\n");
		}
	}


	outportb( (COM2BASE+MCR), DTR );						// set DTR
	delay(100);

	ucTemp1 = inportb(COM2BASE+MSR);
	if( (ucTemp1 & RI) != RI ) {								// RI should be one
		ucError = TRUE;
		if( Debug ) {
			printf("MSR=%02X\n", ucTemp1 );
			printf("Handshake test failed on DTR to RI set to 0\n");
		}
	}

	outportb( (COM2BASE+MCR), 0x00 );			  // clear DTR
	delay(100);

	ucTemp1 = inportb(COM2BASE+MSR);
	if( ucTemp1 & RI ) {											// RI should be zero
		ucError = TRUE;
		if( Debug ) {
			printf("MSR=%02X\n", ucTemp1 );
			printf("Handshake test failed on DTR to RI set to 1\n");
		}
	}

	outportb( P1CFG, ucP1CFGsave );						// restore as found
	outportb( P3CFG, ucP3CFGsave );						// restore as found

	// report error

	if( ucError ) {
		printf("\aDIO1 handshake failure\n");
		return 1;
	} else {
		printf( "OK\n" );
		return 0;                         
	}
}

/****************************************************************************
	Function: test_9400Bus
 Description: using the TS9400, tests the 8 and 16 bit bus
	  Passed: void
	 Returns: int errors (returning >0 means failure) 
		 pre: Options.TS9400 is true
		post: errors > -1 
*****************************************************************************/

int test_9400Bus () {
	unsigned int errors, temp1, temp2, i;
	errors = temp1 = temp2 = i = 0;
	/* First, we check out stuff that doesn't require a 16 bit bus */

	temp1 = inportb(TS9400BASE);		/* 16 bit control lines */

	printf("Checking PC/104 Bus\n");

	/* check bit 3...IORDY */
	if( Debug )	printf("Checking IORDY\n");
	if( (temp1 & 0x08) != 0x08 ) {
		errors++ ;
		if( Debug )
			printf("\aBus Test: IORDY FAILED\n");
	}

	/* Check out OSC signal */
	if( Debug )	printf("Checking OSC\n");
	temp2 = inportb(TS9400BASE) & 0x02 ;

	for( i = 0; i < 400; i++ ) {
		temp1 = inportb(TS9400BASE) & 0x02;	//bit 1 is OSC...should not be stuck
		if( Debug )	printf("original: 0x%x \t read: 0x%x\n", temp2, temp1);
		if( temp1 != temp2 )
			break;
	}
	if( i >= 400 ) {
		errors++;
		if( Debug )
			printf("\aBus Test: OSC signal Failed\n");
	}

	/* Check BCLK signal */
	if( Debug )	printf("Checking BCLK\n");
	temp2 = temp1 = inportb(TS9400BASE) & 0x04;
	for( i = 0; i < 400; i++ ) {
		temp1 = inportb(TS9400BASE) & 0x04;	//bit 2 is BCLK...should not be stuck
		if( temp1 != temp2 ) {
			break;
		}
	}
	if( i >= 100 ) {
		errors++;
		if( Debug )
			printf("\aBus Test: BCLK signal Failed\n");
	}

	/* Now we Check 16 bit bus stuff, if it's enabled */
//	if ( inportb(TS9400BASE+1) & 0x80 == 0x80 ) {		 //16 bit bus enabled
	if( inportb(TS9400BASE+1) > 0x7F ) {		//16 bit bus enabled
		printf("16 Bit Bus Detected\n");
		if( (inportb(TS9400BASE+2) & 0x04) == 0x04 ) {
			if( Debug )	printf("Dip Switch3 ON\n");
			outportb(0x80, 0xAA);
			if( (inportb(TS9400BASE+6)) != 0x55 ) {
				errors++;
				printf("\aBus Test: 16bit Data Bus Failed\n");
				if( Debug ) {
					printf("Output to I/O 0x80: 0xAA: Read in @ 0x19F: 0x%x\n", inportb(0x19F));
				}
			}
		} else {
			if( Debug )	printf("Dip Switch3 OFF \n");
			outportb(0x9D, 0xaa);
			temp2 = inportb(0x9D);
			if( temp2 != 0x55 ) {	  /*byte read in should be complement of byte written in */
				errors++;
				printf("\ABus Test: 16bit Data Bus Failed\n");
				if( Debug ) {
					printf("byte @0x9D: 0x%x\n",temp2);
				}
			}
		}

	} else
		printf("\a16 bit Bus not detected!!!!\n");
	return errors;
}


/****************************************************************************
	Function: test_UserChipSelect
 Description: using the TS9400, tests the UserChipSelect
	  Passed: void
	 Returns: int errors (returning >0 means failure) 
		 pre: Options.TS9400 is true
		post: errors > -1 ...no guarantee of I/O 0x140
*****************************************************************************/

int test_UserChipSelect () {
	int i, errors;
	char temp1, temp2;
/* Write to IO 140, bit0
 * Complement at IO 199, bit0 == success
 * repeat
 */
	errors = temp1 = temp2 = i = 0;
	printf("Testing User Chip Select\n");

	for( i=0; i< 22; i++ ) {
		temp1 = (i%2); 
		outportb (0x140, temp1);
		temp2 = inportb(TS9400BASE) & 1; 
		if( (temp2 & 1) != (~temp1 &1) ) {
			if( Debug ) {
				printf("IO 140,bit0: 0x%02x\n", temp1);
				printf("\tIO 199,bit0: 0x%02x \t i mod 2=%d \n", temp2, (i%2));
			}
			errors++;
		}
	}

	return errors;
}

